  
"
    __________________________________________________
    @Alejandro F. Reimondo 05/09/02
    This proyect sources are property of : Alejandro F. Reimondo
    Its use is NOT legal without written rights.
    Amenabar 3476 Piso: 2 Dto: B.
    Ciudad de Buenos Aires.
    Argentina.
    Tel: (54)-1-703-5586.
    __________________________________________________
"!

ARObject subclass: #FotoIndexBuilder
  instanceVariableNames: 
    ' imagePaths stampsExtension numberOfColumns '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #FotoIndexBuilder) comment: '
	Builder de indice de fotografias para CD.
	@2000 Alejandro F. Reimondo
	aleReimondo@sugarWeb.com
	Example expression:
		FotoIndexBuilder new
    		buildAll:#(Ale Angeles AngelesyAle)
		    at: ''d:\CD-Roms\Fotos''
'  !

ARObject subclass: #AlbumBuilder
  instanceVariableNames: 
    ' parent title numberOfColumns rowsPerTable '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #AlbumBuilder) comment: '
	Builder de indice de fotografias para CD.
	@2000 Alejandro F. Reimondo
	aleReimondo@sugarWeb.com
	Forma de uso:
		AlbumBuilder build: pathName
'    !

ViewManager subclass: #AlbumScanner
  instanceVariableNames: 
    ' dib zoomFactor margin selection mode format pageNumber folder autoScan autoSave autoPrint buildStamps folderText graphPane pageNumberPane dibDetailPane menuButton '
  classVariableNames: ''
  poolDictionaries: 
    ' ColorConstants OperatingSystemConstants '    !


!FotoIndexBuilder methods !
  
build: path
	" Private - Build index page for images at path. "

	^self build: path title: (path reversed upTo: $\) reversed!

build: aDirectory for: masks in: aStream
	" Private - Build index page for images with mask onto aStream in HTML format. "

	|	stamp images stream tuple column 
		startOfTable startOfRow endOfRow
		prologue doPrologue
		relativePath bar aniBar up
	|
	bar := '<img src="global.gif">'.
	aniBar := '<img src="anibar.gif">'.
	up := '<a href="#top"><img src="up.gif" align=right></a>'.
	stream := ReadStream on: aDirectory fullDirName reversed.
	relativePath := ((stream upTo: $\),'\',(stream upTo: $\)) reversed.
	images := SortedCollection new.
	masks do: [:mask|
		images addAll: (aDirectory filesNamed: mask)
	].
	images := images asArray collect: [:fName|
		stamp := self stampOf: fName in: aDirectory.
		(File exists: stamp) ifFalse: [ self error: 'Stamp not found ',stamp ].
		Array with: fName with: stamp
	].

	stream := ReadStream on: images.
	startOfTable := [:what|
		column := 1.
		prologue notNil ifTrue: [
			endOfRow value.
			aStream
				cr;nextPutAll: '</table>';
				cr;nextPutAll: '<center>',aniBar,'</center>',up
		].
		prologue := what.
		prologue notNil ifTrue: [
			aStream
				cr;nextPutAll: '<br><H3><center>',bar,'---',prologue,'---',bar,'</center></H3>';
				cr;nextPutAll: '<table cols=',self numberOfColumns printString,' width=100%>'.
			startOfRow value.
		].
	].
	endOfRow := [ aStream cr;nextPutAll: '</tr>'. column := 1. ].
	startOfRow := [ aStream cr;nextPutAll: '<tr>'. ].
	doPrologue := [:title|
		prologue ~= title ifTrue: [ startOfTable value: title ]
	].

	[stream atEnd] whileFalse: [
		tuple := stream next.
		doPrologue value: (tuple first copyFrom: 1 to: 2).

		aStream cr;nextPutAll: '<td><center>';cr.
		aStream space;
			space;nextPutAll: '<a href="',relativePath,tuple first,'">';
			nextPutAll: '<img src="',relativePath,'Stamps\',tuple last fileNameLessPath,'">';
			nextPutAll: '<br>',tuple first;
			nextPutAll: '</a>'.
		aStream cr;nextPutAll: '</center></td>'.

		column := column + 1.
		column > self numberOfColumns
		ifTrue: [
			endOfRow value.
			startOfRow value.
		].
	].
	endOfRow value.
	doPrologue value: nil.!

build: path title: title
	" Private - Build index page for images at path. "

	| stream |
	stream := WriteStream on: (String new: 2000).
	self build: path title: title in: stream.
	^stream contents!

build: path title: title in: aStream
	" Private - Build index page for images at path. "

	aStream cr;nextPutAll: '<H2>',title,'</H2>'.
	self build: (Directory pathName: path) for: self imagePaths in: aStream.!
  
buildAll: paths at: root
	" Private - Build index page for images at paths inside directory root. "

	| result rootDir |
	rootDir := Directory pathName: root.
	result := String new.
	paths do: [:path|
		result := result , (self build: rootDir fullDirName,path)
	].
	^result!
 
imagePaths
    "Returns the instance var of the receiver
     for external manipulation"

	imagePaths isNil ifTrue: [ self initializeImagePaths ].
    ^imagePaths!

imagePaths: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    imagePaths := anObject!
   
initializeImagePaths
	" Private - Initializes the receiver's imagePaths"

	imagePaths := #( '*.gif' '*.jpg' '*.tif' ).!
  
initializeNumberOfColumns
	" Private - Initializes the receiver's numberOfColumns"

	numberOfColumns := 3.!
  
initializeStampsExtension
	" Private - Initializes the receiver's stampsExtension"

	stampsExtension := '.gif'.!
 
numberOfColumns
    "Returns the instance var of the receiver
     for external manipulation"

	numberOfColumns isNil ifTrue: [ self initializeNumberOfColumns ].
    ^numberOfColumns!

numberOfColumns: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    numberOfColumns := anObject!
 
stampOf: fileName in: directory
	" Privaet - returns the stamp file name of the file fileName in directory."

	^directory fullDirName,'stamps\',(fileName upTo: $.),self stampsExtension!

stampsExtension
    "Returns the instance var of the receiver
     for external manipulation"

	stampsExtension isNil ifTrue: [ self initializeStampsExtension ].
    ^stampsExtension!

stampsExtension: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    stampsExtension := anObject! !



!AlbumBuilder class methods !
  
build: root
	"Build an album from files in directory tree root."

	| directory |
	directory := root isString
		ifTrue: [ Directory pathName: root ]
		ifFalse: [ root ].
	^self new
		build: directory!
 
renameFiles: extension in: root
	"Renombra imagenes moviendo subdirectorios al root..."

	| total ext count count000 convert |
	total := 1. ext := '.',extension.
	convert := [:subdir| count := 1.
		[File exists: root,subdir,'image',count asString,ext] whileTrue: [
	    count000 := total asString.
    	[count000 size < 3] whileTrue: [count000 := '0',count000].
	    File
    	    rename: root,subdir,'image',count asString,ext
        	to: root,count000,ext.
	    count := count + 1. total := total + 1.
		].
	].

	convert value: ''.
	((Directory pathName: root) directories
		collect: [:each| each name,'\' ])
		asSortedCollection do: convert.

	Transcript cr;show: 'Total number of images : ',total printString.!

renameGIFIn: root
	"Renombra imagenes moviendo subdirectorios al root..."

	^self renameFiles: 'gif' in: root!
   
renameJPGIn: root
	"Renombra imagenes moviendo subdirectorios al root..."

	^self renameFiles: 'jpg' in: root!
   
rootFrom: aDirectory
	"Returns the root album directory for aDirectory (or nil)."

	| directory |
	directory := aDirectory isString
		ifTrue: [ Directory pathName: aDirectory ]
		ifFalse: [ aDirectory ].
	^self new	rootFrom: directory! !



!AlbumBuilder methods !
 
build: root
	" Build a page for images at root. "

	| stream subdirectories images |
	root isString
	ifTrue: [ ^self build: (Directory pathName: root) ].

	subdirectories := self subdirectoriesOf: root.
	images := self imageFilesIn: root.
	subdirectories notEmpty | images notEmpty
	ifTrue: [
		stream := root newFile: 'index.htm'.
		[ self
			buildHTMLHeader: root in: stream;
			buildIndices: subdirectories in: stream;
			buildPage: images in: stream;
			buildHTMLFooter: root in: stream;
			yourself.
		] ensure: [ stream close ].
	].!
  
buildHTMLFooter: aDirectory in: stream
	" Private - Build the HTML page header. "

	stream
		cr;nextPutAll: (self footerAt: aDirectory);
		cr;nextPutAll: '</body>';
		cr;nextPutAll: '</html>';
		yourself.!

buildHTMLHeader: aDirectory in: stream
	" Private - Build the HTML page header. "

	| aTitle |
	aTitle := self title isNil
		ifTrue: [ aDirectory pathName fileNameLessPath ]
		ifFalse: [ self title ].
	stream
		nextPutAll: '<html>';
		cr;nextPutAll: '<head><title>', aTitle, '</title></head>';
		cr;nextPutAll: '<body background="',(self imageNamed: 'background.gif'),'">';
		cr;nextPutAll: '<H1>', aTitle, '</H1>';
		cr;nextPutAll: (self descriptionAt: aDirectory);
		cr;
		yourself.!
 
buildIndex: aDirectory in: stream
	" Private - Build the HTML page with index to aDirectory. "

	| label |
	label := aDirectory pathName fileNameLessPath.
	stream
		cr;nextPutAll: '<LI><b><a href="';
		nextPutAll: label,'/index.htm">',label;
		cr;nextPutAll: '</a></b>',(self resumeAt: aDirectory);
		nextPutAll: '</LI>';
		yourself.

	self copy "inherit settings excluding title."
		title: nil;
		parent: self;
		build: aDirectory;
		yourself.!
   
buildIndices: directories in: stream
	" Private - Build the HTML page with index entries to directories. "

	directories isEmpty ifTrue: [ ^self ].

	stream
		cr;nextPutAll: '<p><UL>';
		yourself.

	directories do: [:each|
		self buildIndex: each in: stream
	].

	stream
		cr;nextPutAll: '</UL>';
		yourself.
!
  
buildLinkFor: image in: stream
	" Private - Build a page with links for image. "

	stream "cr;"
		nextPutAll: self textSeparator;
		nextPutAll: (self linkTo: image);
		yourself!
 
buildLinksFor: imageStream in: stream
	" Private - Build a page with links for images in imageStream. "

	"self linksPageSize"1 timesRepeat: [
		imageStream atEnd ifFalse: [
			self buildLinkFor: imageStream next in: stream
		].
	].!

buildPage: images in: stream
	" Private - Build the HTML page with images. "

	images isEmpty ifTrue: [ ^self ].

	(self canUseTablesFor: images)
	ifTrue: [ self buildTablesFor: images in: stream ]
	ifFalse: [ self buildSimplePageFor: images in: stream ].!
 
buildSimplePageFor: imageFiles in: stream
	" Private - Build the HTML page with tables for images. "

	| images |
	images := ReadStream on: imageFiles.
	[images atEnd] whileFalse: [
		self buildLinksFor: images in: stream
	].!
   
buildTableCellFor: image in: stream
	" Private - Build a cell of a table for image. "

	stream "cr;"
		nextPutAll: '<td><center>';
		nextPutAll: (self linkTo: image);
		nextPutAll: '</center></td>';
		yourself!
   
buildTableFor: imageStream in: stream
	" Private - Build a table for images in imageStream. "

	stream
		cr;nextPutAll: '<Table cols=',self numberOfColumns asString, ' width="100%">';
		yourself.

	self rowsPerTable timesRepeat: [
		imageStream atEnd ifFalse: [
			self buildTableRowFor: imageStream in: stream.
		].
	].

	stream cr;nextPutAll: '</table>'!
   
buildTableRowFor: imageStream in: stream
	" Private - Build a row of a table for images in imageStream. "

	stream
		cr;nextPutAll: '<tr>';
		yourself.

	self numberOfColumns timesRepeat: [
		imageStream atEnd ifFalse: [
			self buildTableCellFor: imageStream next in: stream.
		].
	].

	stream cr;nextPutAll: '</tr>'!
 
buildTablesFor: imageFiles in: stream
	" Private - Build the HTML page with tables for images. "

	| images |
	images := ReadStream on: imageFiles.
	[images atEnd] whileFalse: [
		self buildTableFor: images in: stream
	].!
   
canUseTablesFor: images
	" Private - Returns true if aTable can be used to show the images.
	Use a table if at least one stamp is present. "

	images
		detect: [:one| (self stampFor: one) notNil ]
		ifNone: [ ^false ].
	^true!
   
descriptionAt: aDirectory
	" Private - Returns a description string to be embeded. "

	^self stmlFrom: 'description.stml' at: aDirectory!

footerAt: aDirectory
	" Private - Returns a footer string to be embeded. "

	^self stmlFrom: 'footer.stml' at: aDirectory!
   
imageFilesIn: aDirectory
	" Private - Returns the images in aDirectory. "

	| result |
	result := SortedCollection new.
	self imageMasks do: [:mask|
		result addAll: (aDirectory filesNamed: mask)
	].
	^result collect: [:each|
		aDirectory fullDirName, each
	]!
  
imageMasks
	" Private - Returns the masks for image files. "

	^#( '*.jpg' '*.gif' '*.tif' )!

imageNamed: imageName
	" Private - Returns the filename for image imageName. "

	self parent isNil ifTrue: [ ^'images/',imageName ].
	^'../',(self parent imageNamed: imageName)!
   
initializeNumberOfColumns
	" Private - Initializes the receiver's numberOfColumns"

	numberOfColumns := 800 // 210.
	"optimized for stamps of width 210
		and screen size of 800 pixels."!
 
initializeRowsPerTable
	" Private - Initializes the receiver's rowsPerTable"

	rowsPerTable := 5. "max rows per table"!
  
invalidSubdirectories
	" Private - Returns aCollection of subdirectory names considered hidden. "

	^#( 'stamps' 'images' )!
 
linkTo: image
	" Private - Returns a link for image. "

	| imageName reference |
	imageName := image fileNameLessPath.
	reference := self stampFor: image.
	reference := reference isNil
		ifTrue: ['<b>' ,imageName ,'</b> ']
		ifFalse: [
			'<img src="Stamps/'
				,reference fileNameLessPath
				,'" border=0><br>',imageName
		].
	^'<a href="', imageName, '">' ,reference ,'</a>'!

numberOfColumns
    "Returns the instance var of the receiver
     for external manipulation"

	numberOfColumns isNil ifTrue: [ self initializeNumberOfColumns ].
    ^numberOfColumns!

numberOfColumns: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    numberOfColumns := anObject!
 
parent
    "Returns the instance var of the receiver
     for external manipulation"
    ^parent!

parent: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    parent := anObject!
   
resumeAt: aDirectory
	" Private - Returns a resume string to be embeded. "

	^self stmlFrom: 'resume.stml' at: aDirectory!
   
rootFrom: aDirectory
	" Private - Returns the root album directory for aDirectory (or nil). "

	(aDirectory hasDirectoryNamed: 'images') ifTrue: [ ^aDirectory ].
	(aDirectory pathName occurrencesOf: $\) > 1	ifFalse: [ ^nil ].
	^self rootFrom: 
		(Directory pathName: aDirectory drivePathName fileNamePath)!

rowsPerTable
    "Returns the instance var of the receiver
     for external manipulation"

	rowsPerTable isNil ifTrue: [ self initializeRowsPerTable ].
    ^rowsPerTable!

rowsPerTable: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    rowsPerTable := anObject!
   
stampFor: image
	" Private - Returns the stamp of image if present (or nil). "

	| stampName |
	stampName := image fileNamePath,'\Stamps\', image fileNameLessPath.
	^(File exists: stampName) ifTrue: [ stampName ]!
  
stmlContentsFrom: aStream
	" Private - Returns a string obtained from aStream contents.
	HTMLComments are parsed searching for embedded Smalltalk expressions. "

	| output tag script |
	output := WriteStream on: (String new: aStream size).
	[aStream atEnd] whileFalse: [
		output nextPutAll: (aStream upTo: $<).
		aStream peek = $!! ifFalse: [ aStream atEnd ifFalse: [output nextPut: $<] ]
		ifTrue: [ "Comment spected"
			aStream next.	tag := aStream next: 2.
			tag = '--' ifFalse: [ output nextPut: $!!;nextPutAll: tag ]
			ifTrue: [ "Comment found"
				script := aStream nextWord.
				script asLowercase = 'smalltalk'
				ifTrue: [ output nextPutAll: (self valueStmlFrom: aStream) ]
				ifFalse: [ output nextPut: $!!;nextPutAll: tag,script ]
			]
		].
	].
	^output contents!
 
stmlFrom: fileName at: aDirectory
	" Private - Returns a string obtained from an embedable file. "

	| file contents |
	file := (aDirectory
		existingFileNamed: fileName
		ifAbsent: [ ^String new ]) readStream.
	[ contents := self stmlContentsFrom: file.
	] ensure: [ file close ].
	^contents!
  
subdirectoriesOf: aDirectory
	" Private - Returns the subdirectories of aDirectory WITHOUT stamps directory. "

	| sorted |
	sorted := SortedCollection sortBlock: [:a :b| a pathName <= b pathName ].
	sorted addAll: (
		aDirectory directories reject: [:each|
			self invalidSubdirectories
				includes: each pathName fileNameLessPath asLowercase ]).
	^sorted!
 
textSeparator
	" Private - Returns a textSeparator for HTML. "

	^'<br>'!

title
    "Returns the instance var of the receiver
     for external manipulation"
    ^title!
  
title: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    title := anObject!
 
valueStmlFrom: aStream
	" Private - Returns a string obtained from evaluation of Smalltalk code embeded in aStream HTML comment."

	| source compiler |
	source := WriteStream on: String new.
	[	source nextPutAll: aStream upTo: $-.
		aStream peek = $- and: [
			(aStream next;peek) = $>
		]
	 ] whileFalse: [ source nextPut: $- ].
	aStream next.
	compiler := Smalltalk at: #Compiler.
	^compiler
		evaluate: source contents
		in: self species to: self
		notifying: compiler
		ifFail: [^'***Smalltalk Script error***']! !



!AlbumScanner class methods !
 
wbBasicVersion

	"Private - Answer the WindowBuilder Pro version.
		Generated in: Visual Smalltalk 3.1.1 Win32"

	^3.11! !



!AlbumScanner methods !
  
adjustSelection
	" Private - Adjust current selection. "

	selection isNil ifTrue: [ ^self ].
	self dib notNil ifTrue: [
		selection := selection intersect: self dib boundingBox.
	].
	selection := selection truncated.!
   
autoPrint
    "Returns the instance var of the receiver
     for external manipulation"

	autoPrint isNil ifTrue: [ self initializeAutoPrint ].
    ^autoPrint!

autoPrint: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    autoPrint := anObject!
 
autoPrintChanged: aBoolean

	"Callback for the #clicked: event in <Unnamed> [CheckBox].
		(Generated by WindowBuilder Pro)"

	self autoPrint: aBoolean.!
   
autoPrintContents: aPane

	"Callback for the #needsContents event in <Unnamed> [CheckBox].
		(Generated by WindowBuilder Pro)"

	aPane selection: self autoPrint.!
 
autoSave
    "Returns the instance var of the receiver
     for external manipulation"

	autoSave isNil ifTrue: [ self initializeAutoSave ].
    ^autoSave!

autoSave: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    autoSave := anObject!
   
autoSaveChanged: aBoolean

	"Callback for the #clicked: event in <Unnamed> [CheckBox].
		(Generated by WindowBuilder Pro)"

	self autoSave: aBoolean.!
 
autoSaveContents: aPane

	"Callback for the #needsContents event in <Unnamed> [CheckBox].
		(Generated by WindowBuilder Pro)"

	aPane selection: self autoSave.!
   
autoScan
    "Returns the instance var of the receiver
     for external manipulation"

	autoScan isNil ifTrue: [ self initializeAutoScan ].
    ^autoScan!

autoScan: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    autoScan := anObject!
   
autoScanChanged: aBoolean

	"Callback for the #clicked: event in <Unnamed> [CheckBox].
		(Generated by WindowBuilder Pro)"

	self autoScan: aBoolean.!
 
autoScanContents: aPane

	"Callback for the #needsContents event in <Unnamed> [CheckBox].
		(Generated by WindowBuilder Pro)"

	aPane selection: self autoScan.!
   
autoThreshold

    "Callback for the menu item titled 'autoThreshold'.
     (Generated by WindowBuilder)"

	self processWith: [:aProcessableDIB| aProcessableDIB autoThreshold ]!
  
buildAlbum

	"Callback for the #clicked event in <Unnamed> [DrawnButton].
		(Generated by WindowBuilder Pro)"

	| root |
	root := AlbumBuilder rootFrom: self folder.
	root isNil ifTrue: [
		MessageBox warning: 'La carpeta ',self folder pathName,' no esta preparada para ser un album.\No se encontro el directorio principal.\Creelo para luego poder armar el album.'withCrs.
		^nil
	].
	root := root drivePathName.
	AlbumBuilder build: root.
	ProgramManager run: root ,'\index.htm'.!
  
buildStamp
    "Build teh dib stamp if wanted."

	self buildStamps ifFalse: [ ^self ].
    self error: 'Aun no implementado.'!
  
buildStamps
    "Returns the instance var of the receiver
     for external manipulation"

	buildStamps isNil ifTrue: [ self initializeBuildStamps ].
    ^buildStamps!

buildStamps: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    buildStamps := anObject!
 
buildStampsChanged: aBoolean

	"Callback for the #clicked: event in <Unnamed> [CheckBox].
		(Generated by WindowBuilder Pro)"

	self buildStamps: aBoolean.!
   
buildStampsContents: aPane

	"Callback for the #needsContents event in <Unnamed> [CheckBox].
		(Generated by WindowBuilder Pro)"

	aPane selection: self buildStamps.!
 
button1Down: aPane

	"Callback for the #button1Down event in graphPane [GraphPane].
		(Generated by WindowBuilder Pro)"

	self mode == #select ifTrue: [
		^self selection: (aPane mouseLocation extent: 0@0)
	].

	self mode == #move ifTrue: [
		^aPane propertyAt: #startPoint put: aPane mouseLocation
	].!
  
button1Moved: aPane

	"Callback for the #button1Moved event in graphPane [GraphPane].
		(Generated by WindowBuilder Pro)"

	| startPoint |
	self mode == #select ifTrue: [
		self extendSelectionTo: aPane mouseLocation.
		^self
	].

	self mode == #move ifTrue: [
		startPoint := aPane propertyAt: #startPoint.
		self displayGraphPaneAt: aPane mouseLocation - startPoint + self margin.
		^self
	].!
  
clicked: aPoint

	"Callback for the #clicked: event in graphPane [GraphPane].
		(Generated by WindowBuilder Pro)"

	| startPoint |
	self mode == #zoom ifTrue: [
		self zoomFactor: (self zoomFactor * 2 min: 10)
	].

	self mode == #cut ifTrue: [
		self crop.
		graphPane invalidate.
		^self
	].

	self mode == #select ifTrue: [
		^self button1Moved: graphPane
	].

	self mode == #move ifTrue: [
		startPoint := graphPane propertyAt: #startPoint.
		self margin: graphPane mouseLocation - startPoint + self margin.
		^self
	].!
 
confirmFile: pathName

	(File exists: pathName) ifFalse: [
		(MessageBox confirm: 'El archivo ',pathName fileName,' no existe.\Desea crearlo ?' withCrs)
		ifFalse: [ ^false ].
		(File newFile: pathName) close.
	].
	^true!

createViews

	"WARNING!! This method was automatically generated by 
	WindowBuilder. Code you add here which does not conform to 
	the WindowBuilder API will probably be lost the next time 
	you save your layout definition."

	| button1 button2 checkBox1 checkBox2 checkBox3 checkBox4 
	comboBox1 drawnButton1 drawnButton10 drawnButton11 
	drawnButton12 drawnButton13 drawnButton14 drawnButton15 
	drawnButton16 drawnButton2 drawnButton3 drawnButton4 
	drawnButton5 drawnButton6 drawnButton7 drawnButton8 
	drawnButton9 frame1 mainView staticText1 tahoma12Bold xDU 
	yDU |
	
	#generated.
	xDU := (SysFont width + 1 roundTo: 2) / 4.
	yDU := SysFont height / 8.
	mainView := self topPaneClass new.
	
		"Instance Variables"
	dibDetailPane := GroupBox new.
	folderText := StaticText new.
	graphPane := GraphPane new.
	menuButton := DrawnButton new.
	pageNumberPane := CPNumericEditor new.
	
		"Temporary Variables"
	button1 := Button new.
	button2 := Button new.
	checkBox1 := CheckBox new.
	checkBox2 := CheckBox new.
	checkBox3 := CheckBox new.
	checkBox4 := CheckBox new.
	comboBox1 := WBComboBox new.
	drawnButton1 := DrawnButton new.
	drawnButton10 := DrawnButton new.
	drawnButton11 := DrawnButton new.
	drawnButton12 := DrawnButton new.
	drawnButton13 := DrawnButton new.
	drawnButton14 := DrawnButton new.
	drawnButton15 := DrawnButton new.
	drawnButton16 := DrawnButton new.
	drawnButton2 := DrawnButton new.
	drawnButton3 := DrawnButton new.
	drawnButton4 := DrawnButton new.
	drawnButton5 := DrawnButton new.
	drawnButton6 := DrawnButton new.
	drawnButton7 := DrawnButton new.
	drawnButton8 := DrawnButton new.
	drawnButton9 := DrawnButton new.
	frame1 := CP3DFrame new.
	staticText1 := StaticText new.
	
		"Font Definitions"
	tahoma12Bold := Font new
		fromByteArray: #[240 255 255 255 0 0 0 0 0 0 0 0 0 0 0 0 188 2 0 0 0 0 0 0 3 2 1 34 84 97 104 111 109 97 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 ].
	
	mainView
		owner: self;
		setName: 'mainView';
		when: #closed send: #mainViewClosed to: self;
		labelWithoutPrefix: 'Album Scanner';
		noSmalltalkMenuBar;
		backColor: Color gray;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1/2; leftInset: -117 * xDU;
				topRatio: 1/2; topInset: -117 * yDU;
				rightRatio: 1/2; rightInset: -117 * xDU;
				bottomRatio: 1/2; bottomInset: -117 * yDU).
	self addView: mainView.
	
	graphPane
		owner: self;
		setName: 'graphPane';
		when: #button1Down send: #button1Down: to: self with: graphPane;
		when: #button1Moved send: #button1Moved: to: self with: graphPane;
		when: #clicked: send: #clicked: to: self;
		when: #display send: #displayGraphPane to: self;
		when: #doubleClicked send: #doubleClicked: to: self with: graphPane;
		when: #mouseMoved: send: #mouseMoved: to: self;
		when: #needsContents send: #graphPaneContents to: self;
		when: #rightClicked send: #rightClicked: to: self with: graphPane;
		removeHorizontalScrollbarStyle;
		removeVerticalScrollbarStyle;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 0; leftInset: 8 * xDU;
				topRatio: 0; topInset: 68 * yDU;
				rightRatio: 1; rightInset: 22 * xDU;
				bottomRatio: 1; bottomInset: 7 * yDU);
		startGroup;
		noTabStop;
		setPopupMenu: (
			Menu new
				title: '';
				owner: self;
				appendSubMenu: (
					Menu new
						title: '&Procesamiento';
						owner: self;
						appendItem: 'auto&Threshold' selector: #autoThreshold);
				appendSubMenu: (
					Menu new
						title: '&Modo';
						owner: self;
						appendItem: '&mover' selector: #moveMode;
						appendItem: '&zoom' selector: #zoomMode;
						appendItem: '&seleccion' selector: #selectMode)
		);
		font: SysFont.
	mainView addSubpane: graphPane.
	
	pageNumberPane "CPNumericEditor"
		owner: self;
		setName: 'pageNumberPane';
		when: #needsContents send: #pageNumberContents: to: self with: pageNumberPane;
		when: #valueChanged: send: #pageNumberChanged: to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 0; leftInset: 46 * xDU;
				topRatio: 0; topInset: 6 * yDU;
				rightRatio: 0; rightInset: -96 * xDU;
				bottomRatio: 0; bottomInset: -18 * yDU);
		startGroup;
		noTabStop;
		font: SysFont;
		min: 1;
		max: 5000;
		contents: 1.
	mainView addSubpane: pageNumberPane.
	
	staticText1
		owner: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 0; leftInset: 4 * xDU;
				topRatio: 0; topInset: 6 * yDU;
				rightRatio: 0; rightInset: -40 * xDU;
				bottomRatio: 0; bottomInset: -18 * yDU);
		contents: 'Pg. Nro:';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: staticText1.
	
	folderText "StaticText"
		owner: self;
		setName: 'folderText';
		when: #needsContents send: #folderTextContents: to: self with: folderText;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 0; leftInset: 42 * xDU;
				topRatio: 0; topInset: 22 * yDU;
				rightRatio: 1; rightInset: 24 * xDU;
				bottomRatio: 0; bottomInset: -34 * yDU);
		contents: '---Folder---';
		foreColor: Color darkBlue;
		backColor: Color gray;
		startGroup;
		noTabStop;
		font: tahoma12Bold.
	mainView addSubpane: folderText.
	
	button1
		owner: self;
		when: #clicked send: #doScan to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 0; leftInset: 4 * xDU;
				topRatio: 0; topInset: 38 * yDU;
				rightRatio: 0; rightInset: -40 * xDU;
				bottomRatio: 0; bottomInset: -54 * yDU);
		defaultPushButton;
		contents: 'Scanear';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: button1.
	
	button2
		owner: self;
		when: #clicked send: #selectFolder to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 0; leftInset: 4 * xDU;
				topRatio: 0; topInset: 21 * yDU;
				rightRatio: 0; rightInset: -40 * xDU;
				bottomRatio: 0; bottomInset: -37 * yDU);
		contents: 'Carpeta:';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: button2.
	
	checkBox1
		owner: self;
		when: #clicked: send: #autoScanChanged: to: self;
		when: #needsContents send: #autoScanContents: to: self with: checkBox1;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 0; leftInset: 44 * xDU;
				topRatio: 0; topInset: 36 * yDU;
				rightRatio: 0; rightInset: -102 * xDU;
				bottomRatio: 0; bottomInset: -46 * yDU);
		contents: 'Continuamente';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: checkBox1.
	
	comboBox1 "WBComboBox"
		owner: self;
		addClipsiblingsStyle;
		dropDownList;
		when: #changed: send: #formatChanged: to: self;
		when: #needsContents send: #formatListContents: to: self with: comboBox1;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 0; leftInset: 96 * xDU;
				topRatio: 0; topInset: 6 * yDU;
				rightRatio: 0; rightInset: -120 * xDU;
				bottomRatio: 0; bottomInset: -46 * yDU);
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: comboBox1.
	
	drawnButton1
		owner: self;
		when: #clicked send: #zoomMode to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -33/2 * xDU;
				topRatio: 0; topInset: 82 * yDU;
				rightRatio: 1; rightInset: 17/2 * xDU;
				bottomRatio: 0; bottomInset: -90 * yDU);
		cpContents: 'zoom.bmp';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton1.
	
	drawnButton2
		owner: self;
		when: #clicked send: #selectMode to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -33/2 * xDU;
				topRatio: 0; topInset: 92 * yDU;
				rightRatio: 1; rightInset: 17/2 * xDU;
				bottomRatio: 0; bottomInset: -100 * yDU);
		cpContents: 'select.bmp';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton2.
	
	drawnButton3
		owner: self;
		when: #clicked send: #moveMode to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -33/2 * xDU;
				topRatio: 0; topInset: 102 * yDU;
				rightRatio: 1; rightInset: 17/2 * xDU;
				bottomRatio: 0; bottomInset: -110 * yDU);
		cpContents: 'hand.bmp';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton3.
	
	checkBox2
		owner: self;
		when: #clicked: send: #buildStampsChanged: to: self;
		when: #needsContents send: #buildStampsContents: to: self with: checkBox2;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 0; leftInset: 44 * xDU;
				topRatio: 0; topInset: 46 * yDU;
				rightRatio: 0; rightInset: -92 * xDU;
				bottomRatio: 0; bottomInset: -56 * yDU);
		contents: 'Estampillas';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: checkBox2.
	
	menuButton "DrawnButton"
		owner: self;
		setName: 'menuButton';
		when: #clicked send: #openMenu: to: self with: menuButton;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -33/2 * xDU;
				topRatio: 0; topInset: 68 * yDU;
				rightRatio: 1; rightInset: 17/2 * xDU;
				bottomRatio: 0; bottomInset: -76 * yDU);
		cpContents: 'menu.bmp';
		startGroup;
		noTabStop;
		receiveAllWindowMessages;
		setPopupMenu: (
			Menu new
				title: '';
				owner: self;
				appendSubMenu: (
					Menu new
						title: '&Procesar';
						owner: self;
						appendItem: 'flip &horizontal	Alt+H' selector: #flipHorizontally acceleratorString: 'Alt+H';
						appendItem: 'flip &vertical	Alt+V' selector: #flipVertically acceleratorString: 'Alt+V';
						appendItem: 'rotar &90 grados	Alt+9' selector: #rotate90 acceleratorString: 'Alt+9';
						appendItem: 'rotar &270 grados' selector: #rotate270;
						appendSeparator;
						appendItem: 'e&qualizar' selector: #histogramEqualization;
						appendItem: '&contrastar' selector: #enhanceContrast;
						appendItem: '&encontrar bordes' selector: #findEdges;
						appendItem: '&negativo' selector: #invert;
						appendSubMenu: (
							Menu new
								title: '&Filtro';
								owner: self;
								appendItem: 'auto&Threshold' selector: #autoThreshold;
								appendItem: '&mediano' selector: #medianFilter));
				appendSubMenu: (
					Menu new
						title: '&Modo';
						owner: self;
						appendItem: '&zoom	Alt+Z' selector: #zoomMode acceleratorString: 'Alt+Z';
						appendItem: '&seleccionar' selector: #selectMode;
						appendItem: '&mover' selector: #moveMode;
						appendItem: '&cortar	Alt+C' selector: #cutMode acceleratorString: 'Alt+C');
				appendSubMenu: (
					Menu new
						title: '&Archivo';
						owner: self;
						appendItem: '&leer de disco...	Alt+L' selector: #fileOpen acceleratorString: 'Alt+L';
						appendItem: '&guardar	Alt+G' selector: #fileSave acceleratorString: 'Alt+G';
						appendSeparator;
						appendItem: 'guard&ar como...' selector: #fileSaveAs)
		);
		font: SysFont.
	mainView addSubpane: menuButton.
	
	drawnButton4
		owner: self;
		when: #clicked send: #cutMode to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -33/2 * xDU;
				topRatio: 0; topInset: 114 * yDU;
				rightRatio: 1; rightInset: 17/2 * xDU;
				bottomRatio: 0; bottomInset: -122 * yDU);
		cpContents: 'cut.bmp';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton4.
	
	drawnButton5
		owner: self;
		when: #clicked send: #fileSave to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -33/2 * xDU;
				topRatio: 0; topInset: 153 * yDU;
				rightRatio: 1; rightInset: 17/2 * xDU;
				bottomRatio: 0; bottomInset: -161 * yDU);
		cpContents: 'saveImage.bmp';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton5.
	
	drawnButton6
		owner: self;
		when: #clicked send: #fileRun to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -33/2 * xDU;
				topRatio: 0; topInset: 165 * yDU;
				rightRatio: 1; rightInset: 17/2 * xDU;
				bottomRatio: 0; bottomInset: -173 * yDU);
		cpContents: 'run.bmp';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton6.
	
	drawnButton7
		owner: self;
		when: #clicked send: #scanOnce to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -33/2 * xDU;
				topRatio: 0; topInset: 128 * yDU;
				rightRatio: 1; rightInset: 17/2 * xDU;
				bottomRatio: 0; bottomInset: -136 * yDU);
		cpContents: 'scan.bmp';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton7.
	
	drawnButton8
		owner: self;
		when: #clicked send: #folderUp to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -20 * xDU;
				topRatio: 0; topInset: 34 * yDU;
				rightRatio: 1; rightInset: 12 * xDU;
				bottomRatio: 0; bottomInset: -42 * yDU);
		cpContents: 'upFolder.bmp';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton8.
	
	drawnButton9
		owner: self;
		when: #clicked send: #folderNew to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -10 * xDU;
				topRatio: 0; topInset: 34 * yDU;
				rightRatio: 1; rightInset: 2 * xDU;
				bottomRatio: 0; bottomInset: -42 * yDU);
		cpContents: 'newFolder.bmp';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton9.
	
	drawnButton10
		owner: self;
		when: #clicked send: #fileOpen to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -33/2 * xDU;
				topRatio: 0; topInset: 141 * yDU;
				rightRatio: 1; rightInset: 17/2 * xDU;
				bottomRatio: 0; bottomInset: -149 * yDU);
		cpContents: 'fileOpen.bmp';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton10.
	
	drawnButton11
		owner: self;
		when: #clicked send: #buildAlbum to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -10 * xDU;
				topRatio: 0; topInset: 8 * yDU;
				rightRatio: 1; rightInset: 2 * xDU;
				bottomRatio: 0; bottomInset: -16 * yDU);
		cpContents: 'album.bmp';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton11.
	
	drawnButton12
		owner: self;
		when: #clicked send: #printDirectory to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -20 * xDU;
				topRatio: 0; topInset: 22 * yDU;
				rightRatio: 1; rightInset: 12 * xDU;
				bottomRatio: 0; bottomInset: -30 * yDU);
		cpContents: 'print.bmp';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton12.
	
	checkBox3
		owner: self;
		addClipsiblingsStyle;
		when: #clicked: send: #autoSaveChanged: to: self;
		when: #needsContents send: #autoSaveContents: to: self with: checkBox3;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 0; leftInset: 108 * xDU;
				topRatio: 0; topInset: 36 * yDU;
				rightRatio: 0; rightInset: -144 * xDU;
				bottomRatio: 0; bottomInset: -46 * yDU);
		contents: 'Guardar';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: checkBox3.
	
	checkBox4
		owner: self;
		when: #clicked: send: #autoPrintChanged: to: self;
		when: #needsContents send: #autoPrintContents: to: self with: checkBox4;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 0; leftInset: 108 * xDU;
				topRatio: 0; topInset: 46 * yDU;
				rightRatio: 0; rightInset: -144 * xDU;
				bottomRatio: 0; bottomInset: -56 * yDU);
		contents: 'Imprimir';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: checkBox4.
	
	drawnButton13
		owner: self;
		when: #clicked send: #printDIB to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -33/2 * xDU;
				topRatio: 0; topInset: 178 * yDU;
				rightRatio: 1; rightInset: 17/2 * xDU;
				bottomRatio: 0; bottomInset: -186 * yDU);
		cpContents: 'print.bmp';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton13.
	
	drawnButton14
		owner: self;
		when: #clicked send: #fileNew to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -10 * xDU;
				topRatio: 0; topInset: 22 * yDU;
				rightRatio: 1; rightInset: 2 * xDU;
				bottomRatio: 0; bottomInset: -30 * yDU);
		cpContents: 'fileNew.bmp';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton14.
	
	drawnButton15
		owner: self;
		when: #clicked send: #editDescription to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -20 * xDU;
				topRatio: 0; topInset: 46 * yDU;
				rightRatio: 1; rightInset: 12 * xDU;
				bottomRatio: 0; bottomInset: -54 * yDU);
		cpContents: 'description.bmp';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton15.
	
	drawnButton16
		owner: self;
		when: #clicked send: #editFooter to: self;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -10 * xDU;
				topRatio: 0; topInset: 46 * yDU;
				rightRatio: 1; rightInset: 2 * xDU;
				bottomRatio: 0; bottomInset: -54 * yDU);
		cpContents: 'footer.bmp';
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: drawnButton16.
	
	frame1 "CP3DFrame"
		owner: self;
		addClipsiblingsStyle;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 1; leftInset: -20 * xDU;
				topRatio: 0; topInset: 78 * yDU;
				rightRatio: 1; rightInset: 6 * xDU;
				bottomRatio: 0; bottomInset: -124 * yDU);
		in;
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: frame1.
	
	dibDetailPane "GroupBox"
		owner: self;
		setName: 'dibDetailPane';
		when: #needsContents send: #dibDetailContents: to: self with: dibDetailPane;
		framingBlock: 
			(LayoutFrame new
				leftRatio: 0; leftInset: 4 * xDU;
				topRatio: 0; topInset: 58 * yDU;
				rightRatio: 1; rightInset: 4 * xDU;
				bottomRatio: 1; bottomInset: 3 * yDU);
		contents: 'Ultima Imagen Scaneada...';
		foreColor: Color darkRed;
		backColor: Color gray;
		startGroup;
		noTabStop;
		font: SysFont.
	mainView addSubpane: dibDetailPane!

crop
	"Crop the current image to selected rectangle."

	self crop: self cropRectangle.
	"self selection: nil. remove old selection"
	graphPane invalidate.!

crop: aRectangle
	"Crop the current image to aRectangle."

	self error: 'Aun no implementado.'.
!
   
cropRectangle
	"Returns the current crop rectangle."

	| box |
	box := self selection.
	box := box scaleBy: self zoomFactor.
	box := box translateBy: self margin.
	^box!

cutMode

    "Callback for the menu item titled 'cortar'.
     (Generated by WindowBuilder)"

	mode := #cut.!
  
descriptionFileName
	"Returns the current description fileName."

	^self folder drivePathName,'\description.stml'!
   
dib
    "Returns the instance var of the receiver
     for external manipulation"
    ^dib!
  
dib: aDIB
    "Sets the instance var of the receiver with anObject."

    dib := aDIB.
	graphPane invalidate.
	dibDetailPane update.!
  
dibDetailContents: aPane

	"Callback for the #needsContents event in dibDetailPane [GroupBox].
		(Generated by WindowBuilder Pro)"

	aPane contents: (self dib isNil
		ifTrue: [ 'Elija la carpeta y presione "Scanear"' ]
		ifFalse: [ self dib extent printString,' --- ',self dib numberOfColors printString,' colores' ])!
   
dibSupport
    "Returns the DIB file support."

	| support |
	support := format == #jpg ifTrue: [ JPEGImage ]
		ifFalse: [ self error: 'No implementado aun.' ].
	^support!
   
displayGraphPane

	"Callback for the #display event in graphPane [GraphPane].
		(Generated by WindowBuilder Pro)"

	^self displayGraphPaneAt: self margin!
 
displayGraphPaneAt: aMargin
	"Display the graphPane with a aMargin."

	| box |
	self dib isNil ifTrue: [ ^graphPane erase ].
	box := self dib boundingBox scaleBy: self zoomFactor.
	box := box truncated translateBy: aMargin.
	(graphPane boundingBox nonIntersections: box)
	do: [:each| graphPane pen blank: each ].
	self dib
		displayWith: graphPane pen
		from: self dib boundingBox
		to: box.
	self displaySelection.!
   
displaySelection
	" Private - Display the current selection. "

	self displaySelection: self selection.
	
	!
   
displaySelection: aRectangle
	" Private - Display the current selection. "

	aRectangle isNil ifTrue: [ ^self ].
	graphPane pen reverse: aRectangle.
	
	!
 
doScan

	"Callback for the #clicked event in <Unnamed> [Button].
		(Generated by WindowBuilder Pro)"

	[ self scanOnce & self autoScan ] whileTrue.!
   
doubleClicked: aPane

	"Callback for the #doubleClicked event in graphPane [GraphPane].
		(Generated by WindowBuilder Pro)"

	self mode == #zoom ifTrue: [
		self
			clicked: aPane;
			" clicked: aPane; "
			yourself
	].!
  
editDescription

	self editFile: self descriptionFileName!

editFile: pathName

	(self confirmFile: pathName) ifTrue: [
		(Message
			receiver: TextWindow new
			selector: #openOnPathName:
			arguments: (Array with: pathName))
			performDeferred
	].!
  
editFooter

	self editFile: self footerFileName!
  
enhanceContrast

    "Callback for the menu item titled 'contrastar'.
     (Generated by WindowBuilder)"

	self processWith: [:aProcessableDIB| aProcessableDIB enhanceContrast ]!
 
extendSelectionTo: aPoint
	"Adjust the selected rectangle to aPoint."

	| center |
	center := self selection center.
	self selection: (
		(aPoint x < center x ifTrue: [ aPoint x ] ifFalse: [ self selection left ])
		@ (aPoint y < center y ifTrue: [ aPoint y ] ifFalse: [ self selection top ])
		corner: (
			(aPoint x > center x ifTrue: [ aPoint x ] ifFalse: [ self selection right ])
			@ (aPoint y > center y ifTrue: [ aPoint y ] ifFalse: [ self selection bottom ])
		))!

fileName
	"Returns the current fileName."

	^self folder drivePathName,'\',self pageNumber000,'.',self format!
   
fileNew

	"Callback for the #clicked event in <Unnamed> [DrawnButton].
		(Generated by WindowBuilder Pro)"

	self folderUp;folderNew!
  
fileOpen

    "Callback for the menu item titled 'leer de disco...'.
     (Generated by WindowBuilder)"

	| image |
	image := self dibSupport fromUser.
	image notNil ifTrue: [ self dib: image asDIB ].
	image release.!
   
fileRun

	"Callback for the #clicked event in <Unnamed> [DrawnButton].
		(Generated by WindowBuilder Pro)"

	self withDIBDo: [
		(File exists: self fileName) ifTrue: [
			ProgramManager run: self fileName.
		] ifFalse: [ MessageBox warning: 'La imagen se debe guardar antes de hacer esto.' ].
	].!
  
fileSave

	"Callback for the #clicked event in <Unnamed> [DrawnButton].
		(Generated by WindowBuilder Pro)"

	self saveDIB!

fileSaveAs

    "Callback for the menu item titled 'guardar como...'.
     (Generated by WindowBuilder)"

	| aFile |
	aFile := (FileDialog new
		saveTitle: 'Seleccione una carpeta donde guardar la imagen actual.'
		fileName: self fileName) file.
	aFile isNil ifTrue: [ ^nil ].
	self saveDIBTo: aFile.!
 
findEdges

    "Callback for the menu item titled 'encontrar bordes'.
     (Generated by WindowBuilder)"

	self processWith: [:aProcessableDIB| aProcessableDIB findEdges ]!
   
flipHorizontally

    "Callback for the menu item titled 'flip horizontal'.
     (Generated by WindowBuilder)"

	self processWith: [:aProcessableDIB| aProcessableDIB flipHorizontally ]!
  
flipVertically

    "Callback for the menu item titled 'flip vertical'.
     (Generated by WindowBuilder)"

	self processWith: [:aProcessableDIB| aProcessableDIB flipVertically ]!

folder
    "Returns the instance var of the receiver
     for external manipulation"

	folder isNil ifTrue: [ self initializeFolder ].
    ^folder!

folder: aDirectory
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"

    folder := aDirectory.
	self pageNumber: nil.
	folderText notNil ifTrue: [ folderText update ].!
 
folderNew

	"Callback for the #clicked event in <Unnamed> [DrawnButton].
		(Generated by WindowBuilder Pro)"

	| child newFolder |
	child := Prompter
		prompt: 'Nombre de la nueva carpeta (dentro de ',self folder pathName, ') ?'
		default: 'Nueva carpeta'.
	child isNil ifTrue: [ ^nil ].
	newFolder := self folder drivePathName,'\',child.
	Directory create: newFolder.
	self folder: (Directory pathName: newFolder).!

folderTextContents: aPane

	"Callback for the #needsContents event in folderText [StaticText].
		(Generated by WindowBuilder Pro)"

	aPane contents: self folder drivePathName.!
   
folderUp

	"Callback for the #clicked event in <Unnamed> [DrawnButton].
		(Generated by WindowBuilder Pro)"

	| newFolder |
	newFolder := Directory pathName: folder drivePathName fileNamePath.
	CursorManager execute changeFor: [ self folder: newFolder ].!
  
footerFileName
	"Returns the current footer fileName."

	^self folder drivePathName,'\footer.stml'!
  
format
    "Returns the instance var of the receiver
     for external manipulation"

	format isNil ifTrue: [ self initializeFormat ].
    ^format!

format: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    format := anObject!
   
formatChanged: selectedItem

	"Callback for the #changed: event in <Unnamed> [WBComboBox].
		(Generated by WindowBuilder Pro)"

	self format: selectedItem.!
   
formatListContents: aPane

	"Callback for the #needsContents event in <Unnamed> [WBComboBox].
		(Generated by WindowBuilder Pro)"

	aPane
		contents: self formats;
		selection: self format.!
   
formats
	" Private - Returns the receiver's format formats."

	^#( jpg gif tif bmp )!

graphPaneContents

	"Callback for the #needsContents event in graphPane [GraphPane].
		(Generated by WindowBuilder Pro)"

	graphPane
		backColor: Color gray;
		invalidate.

	self setToolTips.!
   
histogramEqualization

    "Callback for the menu item titled 'equalizar'.
     (Generated by WindowBuilder)"

	self processWith: [:aProcessableDIB| aProcessableDIB histogramEqualization ]!
  
initializeAutoPrint
	" Private - Initializes the receiver's autoPrint"

	autoPrint := false.!

initializeAutoSave
	" Private - Initializes the receiver's autoSave"

	autoSave := true.!

initializeAutoScan
	" Private - Initializes the receiver's autoScan"

	autoScan := true.!

initializeBuildStamps
	" Private - Initializes the receiver's buildStamps"

	buildStamps := false.!
  
initializeFolder
	" Private - Initializes the receiver's folder"

	folder := Directory pathName: (
		self profiler at: 'folder'
			ifAbsent: [ Directory current drivePathName ]
		).!

initializeFormat
	" Private - Initializes the receiver's format"

	format := self formats first.!

initializeMargin
	" Private - Initializes the receiver's margin"

	margin := 0@0.!
   
initializeMode
	" Private - Initializes the receiver's mode"

	mode := #select.!
 
initializePageNumber
	" Private - Initializes the receiver's pageNumber"

	pageNumber := 1.
	[File exists: self fileName ]
		whileTrue: [ self pageNumber: pageNumber + 1 ].!
  
initializeZoomFactor
	" Private - Initializes the receiver's zoomFactor"

	zoomFactor := 1.!
 
invert

    "Callback for the menu item titled 'negativo'.
     (Generated by WindowBuilder)"

	self processWith: [:aProcessableDIB| aProcessableDIB invert ]!
 
mainViewClosed

	"Callback for the #closed event in mainView [TopPane].
		(Generated by WindowBuilder Pro)"

	self saveSettings.!
  
margin
    "Returns the instance var of the receiver
     for external manipulation"

	margin isNil ifTrue: [ self initializeMargin ].
    ^margin!

margin: aPoint
    "Sets the instance var of the receiver with anObject."

    margin := aPoint.
	graphPane invalidate.!

medianFilter

    "Callback for the menu item titled 'mediano'.
     (Generated by WindowBuilder)"

	self processWith: [:aProcessableDIB| aProcessableDIB medianFilter ]!
  
mode
    "Returns the instance var of the receiver
     for external manipulation"

	mode isNil ifTrue: [ self initializeMode ].
    ^mode!

mode: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    mode := anObject!
   
mouseMoved: aPoint

	"Callback for the #mouseMoved: event in graphPane [GraphPane].
		(Generated by WindowBuilder Pro)"
!
   
moveMode

	"Callback for the #clicked event in <Unnamed> [DrawnButton].
		(Generated by WindowBuilder Pro)"

	mode := #move.
	self margin: nil.!
  
openMenu: aPane

	"Callback for the #clicked event in <Unnamed> [DrawnButton].
		(Generated by WindowBuilder Pro)"

	aPane doPopupMenuAt: aPane cursorWindowPosition!
  
pageNumber
    "Returns the instance var of the receiver
     for external manipulation"

	pageNumber isNil ifTrue: [ self initializePageNumber ].
    ^pageNumber!

pageNumber: aNumber
    "Sets the instance var of the receiver with anObject."

    pageNumber := aNumber.
	pageNumberPane notNil ifTrue: [ pageNumberPane update ].!
   
pageNumber000
	" Private - Returns the current pageNumber."

	| number |
	number := self pageNumber printString.
	[number size >= 3] whileFalse: [ number := '0',number ].
	^number!
  
pageNumberChanged: anObject

	"Callback for the #valueChanged: event in <Unnamed> [CPNumericEditor].
		(Generated by WindowBuilder Pro)"

	self pageNumber: anObject.!
 
pageNumberContents: aPane

	"Callback for the #needsContents event in <Unnamed> [CPNumericEditor].
		(Generated by WindowBuilder Pro)"

	aPane value: self pageNumber!
 
printDIB
	" Private - Print the current DIB. "

	self printDIB: self dib!

printDIB: aDIB
	" Private - Print the aDIB to default printer. "

	| aPrinter rectangle controlPrintJob aPen factor |
	(aDIB isNil or: [ aDIB width < 5 or: [ aDIB height < 5 ]])	ifTrue: [ ^self ].
	aPrinter := Printer default.
    CursorManager execute changeFor: [
        ( controlPrintJob := aPrinter jobid isNil ) ifTrue: [ aPrinter startPrintJob ].
		aPen := aPrinter pen.
		rectangle := aPen boundingBox insetBy: aPen boundingBox extent // 10.
		"aPen place: rectangle origin; box: rectangle rightBottom."
		factor := rectangle extent / aDIB extent.
		factor := factor x min: factor y.
		factor := aDIB extent * factor // 2.
		rectangle := rectangle center - factor extent: factor * 2.
		aDIB
			displayWith: aPen
			from: aDIB boundingBox
			to: rectangle.
		"aPen place: rectangle origin; box: rectangle rightBottom."
        controlPrintJob ifTrue: [ aPrinter endPrintJob ]
	].!

printDirectory

	"Callback for the #clicked event in <Unnamed> [DrawnButton].
		(Generated by WindowBuilder Pro)"

	| aDirectory image |
	aDirectory := self folder.
	[
		(aDirectory filesNamed: '*.',self format) do: [:fileName|
			dibDetailPane contents: 'Imprimiendo: ',fileName.
			image := self dibSupport
				fromFile: aDirectory drivePathName,'\',fileName.
			self printDIB: image asDIB.
			image release.
		].
		dibDetailPane update.
	] forkAt: 3.!
  
processWith: aBlock
	" Private - Evaluates aBlock as an image processing block. "

	| aProcessableDIB result |
	self withDIBDo: [ 
		aProcessableDIB := ProcessableDIB fromDIB: self dib.
		result := aBlock value: aProcessableDIB
	].
	^result!
   
profiler
	"Returns the profiler of the receiver."

	^WinProfiler new
		pathName: Disk pathName,'\Album.ini';
		applicationName: self class name;
		yourself
		!
  
rightClicked: aPane

	"Callback for the #rightClicked event in graphPane [GraphPane].
		(Generated by WindowBuilder Pro)"

	self mode == #zoom ifTrue: [
		^self zoomFactor: (self zoomFactor / 2 max: 1/10)
	].

	self openMenu: menuButton.!
 
rotate270

    "Callback for the menu item titled 'rotar 270 grados'.
     (Generated by WindowBuilder)"

	self processWith: [:aProcessableDIB| aProcessableDIB rotate: 270 ]!
 
rotate90

    "Callback for the menu item titled 'rotar 90 grados'.
     (Generated by WindowBuilder)"

	self processWith: [:aProcessableDIB| aProcessableDIB rotate: 90 ]!

saveDIB
	" Private - Save the current DIB. "

	self
		saveDIBTo: self fileName;
		saveSettings;
		yourself.!
  
saveDIBTo: aFileName
	" Private - Save the current DIB. "

	| aDIB |
	self withDIBDo: [

		self format == #jpg ifTrue: [
			JPEGImage saveDIB: self dib as: aFileName quality: nil.
			^true
		].

		self format == #bmp ifTrue: [
			aDIB := ProcessableDIB fromDIB: self dib.
			(DrawDIBImage fromDIB: aDIB) saveAs: aFileName.
			^true
		].

		self format == #tif ifTrue: [
			aDIB := ProcessableDIB fromDIB: self dib.
			TIFFImage
				saveDIB: aDIB
				as: aFileName
				compression: (aDIB bitCount = 2
					ifTrue: [ TIFFImage compressionPackBits ]
					ifFalse: [ TIFFImage compressionLzw ]).
			^true
		].

		^self error: 'No implementado Aun.'
	].!
  
saveSettings
	"Save the receiver's settings and current configuration."

	self profiler
		at: 'folder' put: self folder drivePathName;
		yourself!
 
scanOnce

	"Callback for the #clicked event in <Unnamed> [DrawnButton].
		(Generated by WindowBuilder Pro)"

	| hdib aDIB |
	hdib := EZTwainLibrary current
		acquireNative: self mainView handle asParameter.
	aDIB := DIB fromHandle: hdib.
	aDIB isNil ifTrue: [ ^false ]. "aborted"
	self dib: aDIB.
	self autoSave ifTrue: [ self saveDIB ].
	self buildStamps ifTrue: [ self buildStamp ].
	self autoPrint ifTrue: [ self printDIB ].
	self pageNumber: self pageNumber + 1.
	EZTwainLibrary current freeNative: (ExternalHandle fromInteger: hdib) asParameter.
	^true!
  
selectFolder

	"Callback for the #clicked event in <Unnamed> [Button].
		(Generated by WindowBuilder Pro)"

	| aFile |
	aFile := (FileDialog new
		saveTitle: 'Seleccione una carpeta donde guardar las imagenes.'
		fileName: self fileName) file.
	aFile isNil ifTrue: [ ^nil ].
	self folder: (Directory pathName: aFile fileNamePath).!
   
selection
    "Returns the instance var of the receiver
     for external manipulation"
    ^selection!
  
selection: aRectangle
    "Sets the instance var of the receiver with anObject."

	| old |
	old := selection.
    selection := aRectangle.
	self adjustSelection.
	old = selection ifFalse: [
		self
			displaySelection: old;
			displaySelection
	].!
  
selectMode

	"Callback for the #clicked event in <Unnamed> [DrawnButton].
		(Generated by WindowBuilder Pro)"

	mode := #select.
	self selection: nil. "clear old selection"!
 
setToolTips
	" Set the toolTips for all the panes."

	| aName |
	self mainView children asArray do: [:each|
		aName := each name.
		(aName isNil and: [ each isKindOf: DrawnButton ])
		ifTrue: [ aName := each cpContents fileName ].
		each tipText: (self tipTextFor: aName asString)
	]!
   
tipTextFor: aName
	" Returns the toolTip text for aName. "

	(aName isNil or: [ aName = 'nil' ]) ifTrue: [ ^String new ].
	^(self toolTipMap
		detect: [:one| one first = aName ]
		ifNone: [ Array with: aName with: aName,' ?' ]) last!
 
toolTipMap
	" Returns the toolTip map. "

	^#(
	"	(	paneName tipText	)	"
		(	'pageNumberPane' 'Prximo archivo a scanear.'	)
		(	'album' 'Arma un album con todas las imgenes.'	)
		(	'Carpeta:' 'Permite elegir una carpeta donde guardar las pginas scaneadas.'	)
		(	'Scanear' 'Activa el scanner.'	)
		(	'Continuamente' 'Permite scanear una pgina a continuacin de otra (como una fotocopiadora).'	)
		(	'Guardar' 'Guarda automticamente las pginas scaneadas.'	)
		(	'Estampillas' 'Arma una estampilla por cada pgina scaneada. (actualmente esta opcin no est implementada)'	)
		(	'Imprimir' 'Imprime automticamente cada pgina scaneada.'	)
		(	'print' 'Imprime la pgina actual.'	)
		(	'fileNew' 'Abre una nuevo captulo donde scanear. (dentro del mismo libro)'	)
		(	'upFolder' 'Sube un nivel.'	)
		(	'newFolder' 'Permite abrir un nivel dentro del actual.'	)
		(	'description' 'Permite editar una descripcin pra el captulo.'	)
		(	'footer' 'Permite editar un pie de pgina para el captulo.'	)
		(	'graphPane' 'Pgina actual. (ltima scaneada)'	)
		(	'menuButton' 'Men de opciones...'	)
		(	'zoom' 'Aumenta/disminuye el zoom.'	)
		(	'select' 'Seleciona un rea de la pgina.'	)
		(	'hand' 'Permite mover la imagen a una posicin mas cmoda.'	)
		(	'cut' 'Recorta la imagen actual.'	)
		(	'scan' 'Scanea en la pgina actual.'	)
		(	'fileOpen' 'Permite abrir una imagen del disco como si se hubiera scaneado.'	)
		(	'saveImage' 'Graba la imagen actual.'	)
		(	'run' 'Permite editar la imagen con otro programa.'	)
	)!
 
withDIBDo: aBlock
	" Private - Evaluates aBlock as an image processing block. "

	self dib isNil
	ifTrue: [ Terminal bell ]
	ifFalse: [
		CursorManager execute changeFor: aBlock.
		graphPane invalidate.
	].!
 
zoomFactor
    "Returns the instance var of the receiver
     for external manipulation"

	zoomFactor isNil ifTrue: [ self initializeZoomFactor ].
    ^zoomFactor!

zoomFactor: aNumber
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"

    zoomFactor := aNumber.
	graphPane invalidate.!
  
zoomMode

	"Callback for the #clicked event in <Unnamed> [DrawnButton].
		(Generated by WindowBuilder Pro)"

	mode := #zoom.! !
