15/07/00 09:38:50
   
supports: aDIB
    " Private - MUST return true if it's instances can handle aDIB. "

    ^aDIB bitCount = 24! 
for: aDIB
    "Returns an instance of the receiver working on aDIB."

    ^(self supportFor: aDIB) new
        dib: aDIB;
        yourself!  
supportFor: aDIB
    " Private - Returns the class wich instances can handle aDIB."

    ^self allSubclasses
        detect: [:one| one supports: aDIB ]
        ifNone: [ self error: 'Unsupported dib format' ]!   
supports: aDIB
    " Private - MUST return true if it's instances can handle aDIB.
    This method must be refined by subclasses."

    ^false!   
random
    " Returns the random generator class. "

    ^Smalltalk
		at: #NNRandomGenerator
		ifAbsent: [ self error: 'Random number support missing.' ]!
noise: range
    " Apply noice to the receiver. "

    | random |
    random := self class random new.
    self withRGBInRoiDo: [:point :rgb|
        self rgbAt: point put: (
			rgb collect: [:pixel|
				((random nextGaussian * range + pixel) asInteger
					min: 255) max: 0 ])
    ]!
rgbAt: aPoint put: array
    "Sets the rgb value at aPoint."

    | index |
    index :=
        self height - aPoint y - 1
        * self scanLineSize
        + (aPoint x - 1 * 3) + 1.
    "MessageBox message: index asString."
    self bits
		at: index put: (array at: 1) truncated;
		at: index + 1 put: (array at: 2) truncated;
		at: index + 2 put: (array at: 3) truncated!   
pixelAt: aPoint
    "Returns the pixel value at aPoint."

    | rgb |
	rgb := self rgbAt: aPoint.
	^rgb first * 256 + (rgb at: 2) * 256 + rgb last!  
pixelAt: aPoint put: aValue
    "Returns the pixel value at aPoint."

    | rgb value |
	value := aValue asInteger.
	rgb := Array
		with: ((value bitAnd: 16rFF0000) bitShift: -16)
		with: ((value bitAnd: 16r00FF00) bitShift: -8)
		with: (value bitAnd: 16rFF).
	^self rgbAt: aPoint put: rgb!   
rgbAt: aPoint
    "Returns the rgb value at aPoint."

    | index |
    index :=
        self height - aPoint y - 1
        * self scanLineSize
        + (aPoint x - 1 * 3) + 1.
    "MessageBox message: index asString."
    ^self bits copyFrom: index to: index + 2!
postProcessRGBComponentsFrom: start to: stop
    " Private - Switch the rgb values to bgr in the receiver's bits from start to stop."

	| red |
	start to: stop by: 3 do: [:index|
		red := bits at: index.
		bits at: index put: (bits at: index + 2).
		bits at: index + 2 put: red.
	].!   
scanLine: index
    "Returns the bytes for the scanLine number index."

    | start stop |
    start := index * self scanLineSize + 1.
    stop := start + self scanLineSize - 1.
    ^self bits copyFrom: start to: stop!  
bits
    "Returns the instance var of the receiver
     for external manipulation"

    bits isNil ifTrue: [ self initializeBits ].
    ^bits!   
bitmapInfo
    "Returns the instance var of the receiver
     for external manipulation"

    bitmapInfo isNil ifTrue: [ self initializeBitmapInfo ].
    ^bitmapInfo!   
boundingBox
    "Returns the receiver's boundingBox"

    ^self dib boundingBox!   
pixelsInRectangle: aRectangle
    "Returns the bits obtained from aRectangle in the receiver."

    | bytes scanLine gap newScanLineSize |
    newScanLineSize := aRectangle width * self bitCount + 31 // 32 * 4.
    bytes := WriteStream on: (ByteArray new: newScanLineSize * aRectangle height).
	gap := newScanLineSize - (aRectangle width * self bytesPerPixel).
    aRectangle bottom to: aRectangle top by: -1 do: [:index|
        scanLine := self scanLine: self height - index.
        bytes nextPutAll: (scanLine
            copyFrom: aRectangle left * self bytesPerPixel + 1
            to: aRectangle right * self bytesPerPixel).
        gap timesRepeat: [ bytes nextPut: 0 ].
    ].
    ^bytes contents! 
flipHorizontally
    "Flip the receiver horizontally."

    | scanLine start stop |
    0 to: self height - 1 do: [:y|
        start := y * self scanLineSize + 1.
        stop := start + self scanLineSize - 1.
        scanLine := self bits copyFrom: start to: stop.
        self bits
            replaceFrom: start to: stop
            with: scanLine reversed startingAt: 1.
		self postProcessRGBComponentsFrom: start to: stop
    ].!
scanLineSize: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    scanLineSize := anObject! 
height
    "Returns the receiver's height"

    ^self dib height!  
width
    "Returns the receiver's width"

    ^self dib width! 
bitCount
    "Returns the receiver's bitCount"

    ^self dib bitCount!
setBitsTo: pixels extent: extent
    "Sets the dib bits and extent of the receiver's dib."

    self dib setBitsTo: pixels extent: extent.
    self destroyCache.!
pixels
    " Returns the receiver's pixels. "

    | extent result index |
	extent := self extent.
	result := Array new: extent x * extent y.
	index := 1.
	1 to: extent y do: [:y|
		1 to: extent x do: [:x|
			result at: index put: (self pixelAt: x@y).
			index := index + 1.
		].
	].
	^result!
extent
    "Returns the receiver's extent"

    ^self dib extent!  
initializeScanLineSize
    " Private - Initializes the receiver's scanLineSize"

    scanLineSize := self width
        * self bitCount
        + 31 // 32 * 4!  
initializeBits
    " Private - Initializes the receiver's bits"

    bits := self dib bits.!   
postProcessRGBComponentsFrom: start to: stop
    " Private - Switch the rgb values to bgr in the receiver's bits from start to stop."

	^self implementedBySubclass!   
initializeBitmapInfo
    " Private - Initializes the receiver's bitmapInfo"

    bitmapInfo := self dib bitmapInfo.!   
isGrayScale
	"Returns true if the receiver's colors under ROI are gray colors."

	^self hasGrayColorTable or: [ super isGrayScale ]!   
destroyCache
    " Private - Destroy current catched instances. "

    bits := bitmapInfo := scanLineSize := colorTable := nil.!   
noise: range
    " Apply noice to the receiver. "

    | random value |
    random := self class random new.
    self withPixelsInRoiDo: [:point :pixel|
        value := random nextGaussian * range + pixel.
        self pixelAt: point put: value
    ]!  
sizeImage
    " returns the receiver's size in bytes for dib byteArray."

    ^self bitmapInfo sizeImage = 0
        ifTrue: [ self scanLineSize * self height ]
        ifFalse: [ self bitmapInfo sizeImage ].!
dib
    "Returns the instance var of the receiver
     for external manipulation"
    ^dib!
dib: aDIB
    "Sets the instance var of the receiver.)"

    dib := aDIB.
    self destroyCache.! 
colorTable
    "Returns the instance var of the receiver
     for external manipulation"

	colorTable isNil ifTrue: [ self initializeColorTable ].
    ^colorTable!  
flipVertically
    "Flip the receiver vertically."

    | scanLine start stop starth stoph |
    0 to: self height - 1 // 2 do: [:y|
        start := y * self scanLineSize + 1.
        stop := start + self scanLineSize - 1.
        starth := self height - 1 - y * self scanLineSize + 1.
        stoph := starth + self scanLineSize - 1.

        scanLine := self bits copyFrom: start to: stop.
        self bits
            replaceFrom: start to: stop
            with: self bits startingAt: starth;
            replaceFrom: starth to: stoph
            with: scanLine startingAt: 1
    ].! 
hasColorTable
    "Returns true if the receiver uses a color table"

	^self colorTable size > 0!   
hasGrayColorTable
	" Returns true if the receiver's color table are fill with grays. "

	| colors |
	colors := self colorTable.
	colors isEmpty ifTrue: [ ^false ].
	1 to: colors size - 4 by: 4 do: [:index|
		(self isGrayRGB: (colors copyFrom: index to: index + 2))
		ifFalse: [ ^false ].
	].
	^true! 
bytesPerPixel
    "Returns the receiver's bytesPerPixel"

    ^self bitCount / 8!  
scanLineSize
    "Returns the instance var of the receiver
     for external manipulation"

    scanLineSize isNil ifTrue: [ self initializeScanLineSize ].
    ^scanLineSize!   
initializeColorTable
	" Private - Initializes the receiver's colorTable"

	colorTable := self dib bitmapInfo colorTable!   
reset
    "Reset the receiver to an initial state."

    self destroyCache.
    super reset.! 
crop: aRectangle
    "Crop the dib of the receiver to aRectangle."

	| box |
	aRectangle isRectangle ifFalse: [
		"assuming that aRectangle is inset coordinates"
		^self crop: (self boundingBox insetBy: aRectangle)
	].
	box := aRectangle intersect: (self boundingBox insetBy: 1).
    self
		setBitsTo: (self pixelsInRectangle: box)
		extent: box extent!  
grayScale
	"Convert the receiver to grayscale."

	| intensity |
	self withRGBInRoiDo: [:aPoint :rgb|
		intensity := self intensityOf: rgb.
		self rgbAt: aPoint put: (Array with: intensity with: intensity with: intensity)
	]!   
histogramEqualization
    " Perform histogram equaliaztion on the receiver.
    This method must be implemented by subclasses."

    ^self implementedBySubclass! 
isGrayRGB: rgb
	" Private - Returns true if the values in rgb defines a gray color. "

	^(rgb at: 1) = (rgb at: 2) and: [ (rgb at: 1) = (rgb at: 3) ]! 
pixels
    " Returns the receiver's pixels.
    This method must be implemented by subclasses."

    ^self implementedBySubclass! 
withPixelsInRoiDo: aBlock
    "Evaluates aBlock with the pixels in aRectangle."

    ^self withPixelsIn: self roi do: aBlock!  
sharpen
    " Apply sharpen to the receiver.
    This method must be implemented by subclasses."

    ^self implementedBySubclass!
withRGBIn: aRectangle do: aBlock
    "Evaluates aBlock with the RGB values in aRectangle."

    aBlock numberOfArguments = 1
    ifTrue: [
        aRectangle top to: aRectangle bottom do: [:y|
            aRectangle left to: aRectangle right do: [:x|
                aBlock value: (self rgbAt: (x@y) truncated)
            ].
        ].
        ^self
    ].

    aBlock numberOfArguments = 2
    ifTrue: [
        aRectangle top to: aRectangle bottom do: [:y|
            aRectangle left to: aRectangle right do: [:x|
                aBlock value: x@y value: (self rgbAt: (x@y) truncated)
            ].
        ].
        ^self
    ].

    aBlock error: 'Invalid number of arguments.'! 
implementedBySubclass
    " This method is implemented only for browsing.
	Use loacl senders to know what you MUST implement in subclasses."

    ^super implementedBySubclass!   
gamma: value
    "Gamma process on the receiver's color table."

    ^self process: [:i|
		i > 0
		ifTrue: [ ((i / 255.0) ln * value) exp * 255 ]
		ifFalse: [ 0 ]
	]! 
process: unaryBlock
    "Process the image pallete aplying aBlock with no arguments."

    ^self
        process: [:i :ignored| unaryBlock value: i ]
        with: nil! 
initializeRectangleOfInterest
    " Private - Initializes the receiver's rectangleOfInterest"

    rectangleOfInterest := 1@1 corner: self extent - 1! 
add: value
    "Add process on the receiver's color table."

    ^self process: [:i| value + i ]!  
rgbAt: aPoint
    "Returns an array with red green and blue components of pixel at aPoint."

	^self implementedBySubclass! 
withCollumnsIn: aRectangle do: aBlock
    "Evaluates aBlock with the collumns in aRectangle."

    aRectangle left to: aRectangle right do: [:x|
        aBlock value: (self collumnAt: x@aRectangle top length: aRectangle height)
    ].!  
withHSIIn: aRectangle do: aBlock
    "Evaluates aBlock with the RGB values in aRectangle."

    aBlock numberOfArguments = 1
    ifTrue: [
        aRectangle top to: aRectangle bottom do: [:y|
            aRectangle left to: aRectangle right do: [:x|
                aBlock value: (self hsiAt: (x@y) truncated)
            ].
        ].
        ^self
    ].

    aBlock numberOfArguments = 2
    ifTrue: [
        aRectangle top to: aRectangle bottom do: [:y|
            aRectangle left to: aRectangle right do: [:x|
                aBlock value: x@y value: (self hsiAt: (x@y) truncated)
            ].
        ].
        ^self
    ].

    aBlock error: 'Invalid number of arguments.'! 
withHSIInRoiDo: aBlock
    "Evaluates aBlock with the HSI values in aRectangle."

    ^self withHSIIn: self roi do: aBlock!
withRGBInRoiDo: aBlock
    "Evaluates aBlock with the RGB values in aRectangle."

    ^self withRGBIn: self roi do: aBlock!
xMin
    "Private - Minimum x value for 3x3 filter."

    ^self roi left max: 2!   
yMin
    "Private - Minimum y value for 3x3 filter."

    ^self roi top max: 2!
withRowsIn: aRectangle do: aBlock
    "Evaluates aBlock with the rows in aRectangle."

    aRectangle top to: aRectangle bottom do: [:y|
        aBlock value: (self rowAt: aRectangle left@y length: aRectangle width)
    ].!  
pixelAt: aPoint
    " Returns the receiver's the pixel at aPoint.
    This method must be implemented by subclasses."

    ^self implementedBySubclass!   
intensityMax
	" Private - Returns the maximum intensity supported."

	^245!
withIndexTiles: tileExtent do: aBlock
    "Evaluates aBlock with the pixels in tiles of size tileExtent."

    | tile pixels left boundingBox index |
	boundingBox := self roi.
	left := boundingBox left.
	tile := boundingBox leftTop extent: tileExtent.
	index := 1@1.
	[
		pixels := OrderedCollection new: (tileExtent x * tileExtent y) truncated.
		self
			withPixelsIn: (boundingBox intersect: tile)
			do: [:pixel| pixels add: pixel ].
		pixels isEmpty ifFalse: [
			aBlock value: index value: pixels.
		].
		tile := tile translateBy: tile width @ 0.
		index := index + (1 @ 0).
		tile left >= self width ifTrue: [
			tile moveTo: left @ tile bottom.
			tile top >= self height ifTrue: [ ^self ].
			index := 0 @ index y + 1.
		].
	true ] whileTrue.!
hsiAt: aPoint
    "Returns an array with hue saturation and intensity components of pixel at aPoint."

	^self rgb2hsi: (self rgbAt: aPoint)!   
intensityOfRed: red green: green blue: blue
	" Returns the Intensity of and RGB values as an integer value in the 0..255 interval. "

	^red + green + blue // 3!   
applyTable: lut
    " Apply lut table to the receiver.
    This method must be implemented by subclasses."

    ^self implementedBySubclass!  
xor: value
    "Xor process on the receiver's color table."

    ^self process: [:i| i bitXor: value ]!
and: value
    "And process on the receiver's color table."

    ^self process: [:i| i bitAnd: value ]!
collumnAt: aPoint put: aCollection
    " Sets the pixels down collumn from aPoint. "

    1 to: aCollection size do: [:i|
        self
            pixelAt: 0 @ i + aPoint
            put: (aCollection at:i)
    ]!  
colorAt: aPoint
    "Returns an array with red green and blue components of pixel at aPoint."

	| rgb |
	rgb := self rgbAt: aPoint.
	^Color
		red: (rgb at: 1)
		green: (rgb at: 2)
		blue: (rgb at: 3)!  
invert
    "Invert the receiver's color table."

    ^self process: [:i| 255 - i ]!
noise: range
    " Apply noice to the receiver.
    This method must be implemented by subclasses."

    ^self implementedBySubclass! 
autoThreshold
    " AutoThreshold the receiver.
    This method must be implemented by subclasses."

    ^self implementedBySubclass! 
smooth
    " Apply smooth to the receiver.
    This method must be implemented by subclasses."

    ^self implementedBySubclass!  
max
    " Returns the receiver's the largest pixel value.
    This method must be implemented by subclasses."

    ^self implementedBySubclass!   
medianFilter
    " Apply medianFilter the receiver.
    This method must be implemented by subclasses."

    ^self implementedBySubclass! 
collumnAt: index
    " Returns the pixels down collumn number index. "

    ^self collumnAt: index length: self height!
mosaic: tileExtent with: aBlock
    "Evaluates aBlock with each tile in the receiver."

    ^self withTiles: tileExtent do: aBlock!
reset
    "Reset the receiver to an initial state."

    self rectangleOfInterest: nil!
rgbAt: aPoint put: anArray
    "Set pixel with red green and blue components in anArray at aPoint."

	^self implementedBySubclass! 
rowAt: aPoint put: aCollection
    " Sets the pixels allong row from aPoint. "

    1 to: aCollection size do: [:i|
        self
            pixelAt: i @ 0 + aPoint
            put: (aCollection at:i)
    ]!
rowAt: index
    " Returns the pixels along row at index. "

    ^self rowAt: index length: self width!
mosaic: tileExtent
    "Perform mosaic on each tile in the receiver."

	| median boundingBox |
	boundingBox := self boundingBox.
    ^self withTiles: tileExtent do: [:rectangle :pixels|
		median := 0.
		pixels do: [:pixel| median := median + pixel ].
		median := median // pixels size.
		rectangle top to: rectangle bottom do: [:y|
			rectangle left to: rectangle right do: [:x|
				(boundingBox containsPoint: x@y)
				ifTrue: [ self pixelAt: (x@y)truncated put: median ]
			].
		].
	]!
intensityOf: rgb
	" Returns the Intensity of rgb values as an integer value in the 0..255 interval. "

	^self intensityOfRed: (rgb at: 1) green: (rgb at: 2) blue: (rgb at: 3)!
rotate: angle
    " Rotate the receiver by an angle in degrees.
    This method must be implemented by subclasses."

    ^self implementedBySubclass! 
log: value
    "Log process on the receiver's color table."

    | scale |
    scale := value / 255 ln.
    ^self process: [:i| i = 0 ifTrue: [ 0 ] ifFalse: [ i ln * scale ] ]! 
isGrayScale
	"Returns true if the receiver's colors under ROI are gray colors."

	self withRGBInRoiDo: [:rgb|
		(self isGrayRGB: rgb) ifFalse: [ ^false ]
	].
	^true!   
row: index
    " Returns the pixels along row at index. "

    ^self rowAt: index! 
colorAt: aPoint put: aColor
    "Set the image with aColor of pixel at aPoint."

	^self
		rgbAt: aPoint
		put: (Array
			with: aColor red
			with: aColor green
			with: aColor blue)!
findEdges
    " FindEdges in the receiver.
    This method must be implemented by subclasses."

    ^self implementedBySubclass!  
enhanceContrast
    " Enhance Contrast on the receiver.
    This method must be implemented by subclasses."

    ^self implementedBySubclass! 
multiply: value
    "Mult process on the receiver's color table."

    ^self process: [:i| value * i ]!
scaleBy: scale
    " Sets the receiver's extent to current extent * scale."

    ^self resizeTo: ((self extent * scale) truncated max: 1@1)!   
pixelAt: aPoint put: value
    " Sets the receiver's the pixel at aPoint.
    This method must be implemented by subclasses."

    ^self implementedBySubclass!   
fill: color
    "Color Fill the receiver's color table."

    ^self process: [:i| color ]! 
xMax
    "Private - Maximum x value for 3x3 filter."

    ^self roi right min: self width - 2! 
mult: value
    "Mult process on the receiver's color table."

    ^self multiply: value!  
roi
    "Returns the receiver's rectangleOfInterest"

    ^self rectangleOfInterest!   
roi: aRectangle
    "Sets the receiver's rectangleOfInterest"

    ^self rectangleOfInterest: aRectangle!  
yMax
    "Private - Maximum y value for 3x3 filter."

    ^self roi bottom min: self height - 2!   
isValidIntensity: intensity
	" Private - Returns true if intensity is aceptable."

	^intensity between: self intensityMin and: self intensityMax!  
rowAt: aPoint length: length
    " Returns the pixels along row from aPoint. "

    ^(aPoint x to: aPoint x + length)
        collect: [:x| self pixelAt: x @ aPoint y ]! 
intensityMin
	" Private - Returns the minimum intensity supported."

	^10! 
resizeSmoothTo: extent
    " Resize the receiver's contents to extent.
	By default use normal resize algorithm.
    This method must be implemented by subclasses."

    ^self resizeTo: extent! 
withTiles: tileExtent do: aBlock
    "Evaluates aBlock with the pixels in tiles of size tileExtent."

    | tile pixels left boundingBox |
	tileExtent isNumber ifTrue: [
		^self withTiles: tileExtent @ tileExtent do: aBlock
	].
	boundingBox := self roi.
	left := boundingBox left.
	tile := boundingBox leftTop extent: tileExtent.
	[
		pixels := OrderedCollection new: (tileExtent x * tileExtent y) truncated.
		self
			withPixelsIn: (boundingBox intersect: tile)
			do: [:pixel| pixels add: pixel ].
		aBlock numberOfArguments = 2
		ifTrue: [ aBlock value: tile value: pixels ]
		ifFalse: [ aBlock value: pixels ].
		tile := tile translateBy: tile width @ 0.
		tile left >= self width ifTrue: [
			tile moveTo: left @ tile bottom.
			tile top >= self height ifTrue: [ ^self ].
		].
	true ] whileTrue.!  
flipHorizontally
    "Flip the receiver horizontally.
    This method must be implemented by subclasses."

    ^self implementedBySubclass!   
rgb2hsi: rgb
	" Private - Convert a RedGreenBlue color to HueStaturationIntensity color. "

	^self
		rgb2hsi: (rgb at: 1)
		green: (rgb at: 2)
		blue: (rgb at: 3)! 
collumnAt: aPoint length: length
    " Returns the pixels down collumn from aPoint. "

    ^(aPoint y to: aPoint y + length)
        collect: [:y| self pixelAt: aPoint x @ y ]!  
min: min max: max
    " Map the pixels in this image from min-max to 0-255.
    This method must be implemented by subclasses."

    ^self implementedBySubclass! 
or: value
    "Or process on the receiver's color table."

    ^self process: [:i| i bitOr: value ]!   
rgb2hsi: red green: green blue: blue
	" Private - Convert a RedGreenBlue color to HueStaturationIntensity color. "

	| intensity hue saturation sumRGB r g b x y o3 t3 arg |
	sumRGB := (red + green + blue) asFloat.

	intensity := sumRGB / 3.
	(self isValidIntensity: intensity) ifFalse: [ ^nil ].

	o3 := 1 / 3 asFloat.
	t3 := 2 / 3 asFloat.

	r := red / sumRGB.
	g := green / sumRGB.
	b := blue / sumRGB.

	x := ((r - o3) squared + (g - o3) squared + (b - o3) squared) sqrt.
	x = 0.0 ifTrue: [ ^nil ].
	y := t3 * (r - o3) - o3 * (g - o3) - o3 * (b - o3).
	arg := (y / (x * t3 sqrt) max: -1) min: 1.

	hue := arg arcCos.
	hue > Float pi ifTrue: [ hue := Float pi * 2 - hue ].

	saturation := 1 - (((r min: g) min: b) * 3).

	^Array
		with: hue
		with: saturation
		with: intensity! 
process: aBlock with: argument
    "Process the image pallete aplying aBlock with argument."

    | lut |
    lut := (0 to: 255) collect: [:i|
        ((aBlock value: i value: argument)
            truncated min: 255) max: 0
    ].
    self applyTable: lut.!
flipVertically
    "Flip the receiver vertically.
    This method must be implemented by subclasses."

    ^self implementedBySubclass!   
rectangleOfInterest
    "Returns the instance var of the receiver
     for external manipulation"

    rectangleOfInterest isNil ifTrue: [ self initializeRectangleOfInterest ].
    ^rectangleOfInterest!   
rectangleOfInterest: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    rectangleOfInterest := anObject!   
min
    " Returns the receiver's the smallest pixel value.
    This method must be implemented by subclasses."

    ^self implementedBySubclass!  
collumn: index
    " Returns the pixels down collumn number index. "

    ^self collumnAt: index!  
histogram
    " Returns the receiver's histogram.
    This method must be implemented by subclasses."

    ^self implementedBySubclass!   
applyMask: mask
    " Apply a mask to the receiver.
    This method must be implemented by subclasses."

    ^self implementedBySubclass! 
resizeTo: extent
    " Resize the receiver's contents to extent.
    This method must be implemented by subclasses."

    ^self implementedBySubclass!
withPixelsIn: aRectangle do: aBlock
    "Evaluates aBlock with the pixels in aRectangle."

    aBlock numberOfArguments = 1
    ifTrue: [
        aRectangle top to: aRectangle bottom do: [:y|
            aRectangle left to: aRectangle right do: [:x|
                aBlock value: (self pixelAt: (x@y) truncated)
            ].
        ].
        ^self
    ].

    aBlock numberOfArguments = 2
    ifTrue: [
        aRectangle top to: aRectangle bottom do: [:y|
            aRectangle left to: aRectangle right do: [:x|
                aBlock value: x@y value: (self pixelAt: (x@y) truncated)
            ].
        ].
        ^self
    ].

    aBlock error: 'Invalid number of arguments.'!  
supports: aDIB
    " Private - MUST return true if it's instances can handle aDIB. "

    ^aDIB bitCount = 8!  
threshold: level
    " Threshold the receiver to level."

    1 to: self bits size do: [:i|
        bits at: i put: (
            ((bits at: i) bitAnd: 16rFF) <= level
            ifTrue: [ 0 ] ifFalse: [ 16rFF ]
        )
    ].!
rgbAt: aPoint put: anArray
    "Set pixel with red green and blue components in anArray at aPoint."

    | index i tuple |
    i := (0 to: self colorTable size // 4 - 1)
        detect: [:k|
            tuple := self colorTable copyFrom: k * 4 + 1 to: k * 4 + 3.
            tuple asArray = anArray  ]
        ifNone: [ self error: 'Invalid color.' ].
    index :=
        self height - aPoint y
        * self scanLineSize
        + aPoint x.
    self bits at: index put: i.! 
pixelAt: aPoint
    "Returns the pixel value at aPoint."

    | index |
    index :=
        self height - aPoint y
        * self scanLineSize
        + aPoint x.
    "MessageBox message: index asString."
    ^self bits at: index!  
resizeSmoothTo: extent
    " Resize the receiver's dib to extent."

	| pixels newScanLineSize index median |
    (extent x > self width or: [ extent y > self height ])
	ifTrue: [ ^self resizeTo: extent ].

	"slow resize is only applicable to reductions."
    newScanLineSize := extent x * self bitCount + 31 // 32 * 4.
    pixels := ByteArray new: newScanLineSize * extent y.

	self
		withIndexTiles: self extent / extent
		do: [:point :values |
			median := 0.
			values do: [:each| median := median + each ].
			median := median // values size.
			index := extent y - point y * newScanLineSize + point x.
			pixels at: index put: ((median max: 0) min: 255) truncated
		].

    self setBitsTo: pixels extent: extent.!   
privateReduceBy: aPoint
    " Private - Reduce the image in the receiver by aPoint."

    | x y length newExtent stream newScanLineSize gap median |
	length := self width / aPoint x.
	x := length = length truncated
		ifTrue: [ length truncated ]
		ifFalse: [ length truncated + 1 ].
	length := self height / aPoint y.
	y := length = length truncated
		ifTrue: [ length truncated ]
		ifFalse: [ length truncated + 1 ].
	newExtent := x @ y.
	self extent = newExtent ifTrue: [ ^self ].
    newScanLineSize := newExtent x * self bitCount + 31 // 32 * 4.
    stream := WriteStream on: (ByteArray new: newScanLineSize * newExtent y).
	gap := newScanLineSize - newExtent x.
	self withTiles: self extent / newExtent do: [:tile :pixels|
		median := 0.
		pixels do: [:pixel| median := median + pixel ].
		median := median // pixels size.
		stream nextPut: ((median max: 0) min: 255).
		tile right > self width ifTrue: [ gap timesRepeat: [ stream nextPut: 0 ] ].
	].
    self setBitsTo: stream contents extent: newExtent.!   
histogramEqualization
    " Perform histogram equalization on the receiver."

    | histogram sum scale lut |
    histogram := self histogram.
    sum := 0.
    histogram do: [:each| sum := sum  + each ].
	sum = 0 ifTrue: [ ^self ].
    scale := 255.0 / sum.
    sum := 0.
    lut := Array new: 256.
    1 to: lut size do:[:i|
        sum := sum + (histogram at: i).
        lut at: i put: (sum * scale) truncated.
    ].
    self applyTable: lut.!   
applyTable: lut
    " Apply lut table to the receiver. "

    self withPixelsInRoiDo: [:point :pixel|
        self pixelAt: point put: (lut at: pixel + 1)
    ]!
rotate: angle
    " Rotate the receiver by an angle in degrees."

    |    pixels center width height
        scale angleRadians
        ca sa s t u
        index
    |
    angle \\ 360 = 0 ifTrue: [ ^self ].
    pixels := self bits copy.
    center := self roi center.
    width := self scanLineSize.
    height := self height.
    scale := 1024.
    "
    Convert from degrees to radians and calculate cos and sin of angle
    Negate the angle to make sure the rotation is clockwise
    "
    angleRadians := angle degreesToRadians"negated / (180 / Float pi)".
    ca := (angleRadians cos * scale) truncated.
    sa := (angleRadians sin * scale) truncated.
    u := center y @ center x negated * sa - (center * ca).
    self roi top to: self roi bottom do:[:y|
        index := y - 1 * width + self roi left.
        t := sa negated @ ca * y + u // scale + center.
        self roi left to: self roi right do: [:x|
        s := ca @ sa * x // scale + t.
        ((0@0 extent: width @ height) containsPoint: s)
        ifTrue: [ self bits at: index put: (pixels at: s y * width + s x) ]
        ifFalse: [ self bits at: index put: 127 ].
        index := index + 1.
        ].
    ].! 
findMedian: values
    " Private - Finds the 5th largest of 9 values. "

    | max mj |
    1 to: 4 do: [:i|
        max := 0.
        mj := 1.
        1 to: 9 do: [:j|
            (values at: j) > max
            ifTrue: [
                max := values at: j.
                mj := j.
            ].
        ].
        values at: mj put: 0.
    ].
    max := 0.
    1 to: 9 do: [:j|
        (values at: j) > max
        ifTrue: [ max := values at: j ].
    ].
    ^max!   
postProcessRGBComponentsFrom: start to: stop
    " Private - Switch the rgb values to bgr in the receiver's bits from start to stop."

	"do nothing because the receiver is paletized..."! 
reduceBy: factor
    "Reduce the image in the receiver by factor."

	| default |
	default := [ self resizeTo: (self extent / factor) truncated ].
"
	(factor isNumber and: [ factor isInteger not ])
	ifTrue: [ ^default value ].
"
	factor = 1 ifTrue: [ ^self ].
	factor = 2 ifTrue: [ ^self reduceBy2 ].
	factor isNumber ifTrue: [
		factor < 1 ifTrue: [ ^default value ].
		^self reduceBy: factor @ factor
	].
"
	(factor x < 1 or: [ factor y < 1 ])
	ifTrue: [ ^self error: 'Can''t reduce to factor<1' ].
"
	^self privateReduceBy: factor!  
reduceBy2
    "Reduce the image in the receiver by 2."

    | newExtent scanLine1 scanLine2 stream median gap |
    newExtent := self extent - 1 // 2.
    gap := newExtent x * self bitCount + 31 // 32 * 4 - newExtent x.
    stream := WriteStream on: (ByteArray new: newExtent x + gap * newExtent y).
    0 to: newExtent y - 1 do: [:index|
        scanLine1 := self scanLine: index * 2 + 1.
        scanLine2 := self scanLine: index * 2 + 2.
        0 to: newExtent x - 1 do: [:x|
            median := scanLine1 at: x * 2 + 1.
            median := median + (scanLine1 at: x * 2 + 2).
            median := median + (scanLine2 at: x * 2 + 1).
            median := median + (scanLine2 at: x * 2 + 2).
            stream nextPut: median // 4.
        ].
        gap timesRepeat: [ stream nextPut: 0 ].
    ].
    self setBitsTo: stream contents extent: newExtent.!
scanLinesFrom: start to: stop
    "Returns the scanLines from start to stop."

    ^(start to: stop) collect: [:index| self scanLine: index ]! 
autoThreshold
    " AutoThreshold the receiver.
    Iterative thresholding technique, described originally by Ridler & Calvard in
    'Picture Thresholding Using an Iterative Selection Method', IEEE transactions
    on Systems, Man and Cybernetics, August, 1978."

    | level histogram result min max movingIndex sum1 sum2 sum3 sum4 current |
    histogram := self histogram.
    histogram
        at: 1 put: 0; "set to zero so erased areas won't be included in the threshold"
        at: histogram size put: 0;
        yourself.
    min := 1.max := histogram size.
    [ (histogram at: max) = 0 ] whileTrue: [
        max := max - 1.
        max = min ifTrue: [ ^self threshold: 128 ].
    ].
    [ (histogram at: min) = 0 ] whileTrue: [
        min := min + 1.
        min >= max ifTrue: [ ^self threshold: 128 ].
    ].
    movingIndex := min.
    [
        sum1 := sum2 := sum3 := sum4 := 0.
        min to: movingIndex do: [:index|
            current := histogram at: index.
            sum1 := index * current + sum1.
            sum2 := current + sum2.
        ].
        movingIndex + 1 to: max do: [:index|
            current := histogram at: index.
            sum3 := index * current + sum3.
            sum4 := current + sum4.
        ].
        result := (sum1 / sum2 / 2.0) + (sum3 / sum4 / 2.0).
        movingIndex := movingIndex + movingIndex.

        movingIndex + 1 <= result
        and: [ max - 1 >= movingIndex ]
    ] whileTrue.
    ^self threshold: result rounded!  
findEdges
    " FindEdges in the receiver. "

    |    pixels lineSize offset start
        sum1 sum2 sum
        p1 p2 p3
        p4 p5 p6
         p7 p8 p9
    |
    lineSize := self scanLineSize.
    pixels := self bits copy.
    self yMin to: self yMax do: [:y|
        start := y - 1 * lineSize + self xMin.
        offset := start + 1.
        p2 := pixels at: offset - lineSize - 2.
        p3 := pixels at: offset - lineSize - 1.
        p5 := (pixels at: offset) "- 2".
        p6 := pixels at: offset - 1.
        p8 := pixels at: offset + lineSize - 2.
        p9 := pixels at: offset + lineSize - 1.
        offset := start.
        self xMin to: self xMax do: [:x|
            p1 := p2. p2 := p3.
            p3 := pixels at: offset - lineSize + 1.
            p4 := p5. p5 := p6.
            p6 := pixels at: offset + 1.
            p7 := p8. p8 := p9.
            p9 := pixels at: offset + lineSize + 1.
            sum1 := p1 + (2 * p2) + p3 - p7 - (2 * p8) - p9.
            sum2 := p1 + (2 * p4) + p7 - p3 - (2 * p6) - p9.
            sum := (sum1 squared + sum2 squared) sqrt min: 255.
            self bits at: offset put: sum truncated.
            offset := offset + 1.
        ].
    ].!  
enhanceContrast
    " Enhance Contrast on the receiver. "

    | histogram sum threshold i found min max delta |
    histogram := self histogram.
    sum := 0.    histogram do: [:each| sum := sum + each ].
    threshold := sum / 5000.
    i := 0.found := false.
    [    i := i + 1.
        found := (histogram at: i) > threshold.
        found not and: [ i < 256 ]
    ] whileTrue.
    min := i.
    i := 257.
    [    i := i - 1.
        found := (histogram at: i) > threshold.
        found not and: [ i > 1 ]
    ] whileTrue.
    max := i.
    max > min
    ifTrue: [
        delta := max - min.
        self roi: nil.
        (min <= 5 and: [ delta >= 250 ])
        ifTrue: [ self add: -12;multiply: 1 ]
        ifFalse: [ self add: min negated;multiply: 256/ delta
        ]
    ]!   
histogram
    " Returns the receiver's histogram."

    | histogram |
    histogram := Array new: 256.
    histogram atAllPut: 0.
    self withPixelsIn: self roi do: [:pixel|
        histogram at: pixel + 1 put: (histogram at: pixel + 1) + 1
    ].
    ^histogram! 
medianFilter
    " Apply medianFilter the receiver. "

    |    pixels lineSize offset start
        sum1 sum2 sum values
        p1 p2 p3
        p4 p5 p6
         p7 p8 p9
    |
    lineSize := self scanLineSize.
    pixels := self bits copy.
    values := Array new: 9.
    self yMin to: self yMax do: [:y|
        start := y - 1 * lineSize + self xMin.
        offset := start + 1.
        p2 := pixels at: offset - lineSize - 2.
        p3 := pixels at: offset - lineSize - 1.
        p5 := (pixels at: offset) "- 2".
        p6 := pixels at: offset - 1.
        p8 := pixels at: offset + lineSize - 2.
        p9 := pixels at: offset + lineSize - 1.
        offset := start.
        self xMin to: self xMax do: [:x|
            p1 := p2. p2 := p3.
            p3 := pixels at: offset - lineSize + 1.
            p4 := p5. p5 := p6.
            p6 := pixels at: offset + 1.
            p7 := p8. p8 := p9.
            p9 := pixels at: offset + lineSize + 1.

            values
                at: 1 put: p1; at: 2 put: p2; at: 3 put: p3;
                at: 4 put: p4; at: 5 put: p5; at: 6 put: p6;
                at: 7 put: p7; at: 8 put: p8; at: 9 put: p9;
                yourself.

            self bits at: offset put: (self findMedian: values) truncated.
            offset := offset + 1.
        ].
    ].!   
pixelAt: aPoint put: value
    "Sets the pixel value at aPoint to value."

    | index |
    index :=
        self height - aPoint y
        * self scanLineSize
        + aPoint x.
    "MessageBox message: index asString."
    ^self bits at: index put: ((value max: 0) min: 255) truncated!
resizeTo: extent
    " Resize the receiver to extent."

    |    sourceCenter destinationCenter
        scale pixels newScanLineSize
        index xs ys
    |
    sourceCenter := self roi center truncated.
    destinationCenter := extent // 2.
    scale := extent / self roi extent * 1.0.
    newScanLineSize := extent x * self bitCount + 31 // 32 * 4.
    pixels := ByteArray new: newScanLineSize * extent y.
    1 to: extent y do: [:y|
        index := y - 1 * newScanLineSize + 1.
        ys := y - destinationCenter y // scale y + sourceCenter y.
        1 to: newScanLineSize do: [:x|
            xs := x - destinationCenter x // scale x + sourceCenter x.
            pixels
                at: index
                put: (self bits
                    at: ((ys - 1 * self scanLineSize + xs
                        max: 1) min: self bits size)).
            index := index + 1.
        ].
    ].

    self setBitsTo: pixels extent: extent.!   
rgbAt: aPoint
    "Returns the pixel value at aPoint."

    | index i |
    index :=
        self height - aPoint y
        * self scanLineSize
        + aPoint x.
    "MessageBox message: index asString."
    i  := (self bits at: index) * 4 + 1.
    ^self colorTable copyFrom: i to: i + 3!  
@ aDirectory
	"Returns all the images found from aDirectory (recursivelly)."

	| result |
	aDirectory isString ifTrue: [
		^self @ (Directory pathName: aDirectory)
	].
	result := OrderedCollection new.
	self imagesAt: aDirectory do: [:each| result add: each ].
	^result!   
load: anInstance fromFile: fileName
    " Private - Load anInstance internal properties from file named fileName."

    ^anInstance fromDIB: (DIB fromFile: fileName)! 
imagesAt: aDirectory do: aBlock
	"Evaluates aBlock with all the images from the directory (recursivelly)."

	^(self fileNamesIn: aDirectory recursive: true)
		do: [:fileName| aBlock value: (self fromFile: fileName) ]! 
fileNamesIn: aDirectory
	" Private - Returns all the fileNames of files inside aDirectory."

	^self fileNamesIn: aDirectory recursive: false!  
fileNamesIn: aDirectory recursive: recursive
	" Private - Returns all the fileNames of files inside aDirectory. "

	| fileNames current |
	aDirectory exists ifFalse: [
		^aDirectory error: 'Invalid directory'
	].
	fileNames := aDirectory fileNames collect: [:each|
		current := aDirectory drivePathName,'\',each.
		(self isValidFileName: current) ifTrue: [current]
	].
	recursive ifTrue: [
		aDirectory subdirectories do: [:array|
			fileNames := fileNames,
				(self fileNamesIn: (Directory pathName: array first) recursive: true)
		]
	].
	^fileNames select: [:each| each notNil ]!   
isValidFileName: pathName
	" Private - Must return true if a file named pathName can be accepted as a valid image file.
	The default answer only validates file extension.
	This method can be refined by subclasses."

	^self isValidFileExtension: pathName fileExtension trimBlanks!  
fromDIB: aDIB
    "Returns anew instance of the receiver from aDIB if posible, else generates anError."

    ^self new fromDIB: aDIB!  
isValidFileExtension: fileExtension
	" Private - Must return true if a file with extension fileExtension can be accepted as a valid image file."

	^fileExtension asLowerCase = 'bmp'! 
fromFile: bitmapFile
    "Create a DIB bitmap from bitmapFile."

    | image |
    image := self new. "defer image loading...
                                    super fromFile: bitmapFile.
                                    "
    image fileName: bitmapFile.
    ^image!   
grayScale
    "Convert receiver's colors to gray."

    self processor grayScale!  
printOn: aStream
	"Append the ASCII representation of the receiver to aStream."

   super printOn: aStream.
	aStream nextPutAll: ' on ',self fileName asString!   
histogramEqualization
    "Apply histogramEqualization to the receiver."

    self processor histogramEqualization!
asBitmap
    "Convert from DIB bitmap to DDB bitmap."

    self mustBeLoaded.
    ^super asBitmap!
sharpen
    " Apply sharpen to the receiver. "

    self processor sharpen!
initializeProcessor
    " Private - Initializes the receiver's processor"

    processor := DIBProcessor for: self.!   
copy
    "Returns a copy of the receiver."

    ^super copy
        bitmapInfo: self bitmapInfo deepCopy;
        bits: self bits copy;
		processor: nil;
        yourself!
processor
    "Returns the instance var of the receiver
     for external manipulation"

    processor isNil ifTrue: [ self initializeProcessor ].
    ^processor!   
processor: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    processor := anObject!   
gamma: value
    "Gamma process on the receiver's color table."

	self processor gamma: value! 
fromDIB: aDIB
    "Sets the receiver from aDIB if posible, else generates anError."

    self
		bitmapInfo: aDIB bitmapInfo;
		bits: aDIB bits;
		yourself! 
fileName: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    fileName := anObject! 
scanLine: index
    "Returns the bytes for the scanLine number index."

    ^self processor scanLine: index!   
pixelAt: aPoint
    "Returns the pixel value at aPoint."

    ^self processor pixelAt: aPoint! 
bitmapInfo
    "Answer the bitmapInfo structure."

    self mustBeLoaded.
    ^bitmapInfo!
bits
    "Answer the receiver's bits."

    self mustBeLoaded.
    ^bits! 
bitCount
    "Answer the bitCount field of bitmapInfo structure."

    self mustBeLoaded.
    ^super bitCount!
mustBeLoaded
    " Private - Try to load all the receiver onto memory for processing.
    Generates an error in can't be loaded from file."

    bits isNil ifFalse: [ ^self ].
    self class load: self fromFile: self fileName.!  
xor: value
    "Xor process on the receiver's color table."

    self processor xor: value!
and: value
    "Xor process on the receiver's color table."

    self processor and: value!
outputToFile: aFileName
        "Output the receiver in DIB format to aFileName."
    | aFileStream |
    aFileStream := ( File fromPath: aFileName ) binaryWriteStream.
    self outputToFileStream: aFileStream.
    aFileStream close!
invert
    "Invert the receiver"

	self processor invert!  
noise: range
    " Apply noice to the receiver. "

    self processor noise: range!
autoThreshold
    "AutoThreshold the receiver."

    self processor autoThreshold! 
autoThreshold: aRectangle
    "AutoThreshold based on aRectangle in the receiver."

    self processor
        roi: aRectangle;
        autoThreshold!   
smooth
    " Apply smooth to the receiver. "

    self processor smooth!   
medianFilter
    "Apply medianFilter to the receiver."

    self processor medianFilter!   
threshold: level
    " Threshold the receiver to level."

    self processor threshold: level! 
width
    "Answer receiver's width."

    self mustBeLoaded.
    ^super width!
height
    "Answer receiver's height."

    self mustBeLoaded.
    ^super height! 
asDIB
	"Returns the receiver as a raw DIB."

	^DIB new
        bitmapInfo: self bitmapInfo deepCopy;
        bits: self bits copy;
        yourself!
mosaic: tileExtent
    "Perform mosaic on each tile in the receiver.
	tileExtent can be a point or a number."

	self processor mosaic: tileExtent!
rotate: angle
    " Rotate the receiver by an angle in degrees."

    self processor rotate: angle!
log: value
    "Log process on the receiver's color table."

	self processor log: value!   
isGrayScale
    "Returns true if the receiver's colors under ROI are gray colors."

    ^self processor isGrayScale!   
reduceBy: factor
    "Reduce the receiver by factor."

    self processor reduceBy: factor!
noise: range to: aRectangle
    " Apply noice to aRectangle in the receiver. "

    self processor
        roi: aRectangle;
        noise: range!
findEdges
    "Apply findEdges to the receiver."

    self processor findEdges!
enhanceContrast
    "Apply enhanceContrast to the receiver."

    self processor enhanceContrast!  
multiply: value
    "Mult process on the receiver's color table."

    self processor multiply: value! 
outputToFileStream: aFileStream
    "Output the receiver in DIB format to aFileStream."

    | fileHeader |
    fileHeader := WinBitmapFileHeader new.
    fileHeader bfType: 'BM';
        bfSize: 14 + self bitmapInfo contents size + self bits size;
        reserved1: 0;
        reserved2: 0;
        offBits: 14 + self bitmapInfo contents size.
    aFileStream nextPutAll: fileHeader contents.
    aFileStream putBytesFrom: self bitmapInfo contents.
    aFileStream putBytesFrom: self bits!   
pixelAt: aPoint put: value
    "Sets the pixel value at aPoint to value."

    self processor pixelAt: aPoint put: value!  
scanLineSize
    "Returns the receiver's scan line size."

    ^self processor scanLineSize!   
crop: aRectangle
    "Crop the receiver at aRectangle."

    self processor crop: aRectangle!  
bitmapInfo: aBitmapInfo
    "Sets the receiver's bitmapInfo."

    processor := nil.
    ^super bitmapInfo: aBitmapInfo!  
resizeSmoothTo: extent
    " Resize the receiver to extent."

    self processor resizeSmoothTo: extent!   
fileName
    "Returns the instance var of the receiver
     for external manipulation"
    ^fileName!  
flipHorizontally
    "Flip the receiver horizontally."

    self processor flipHorizontally!   
setBitsTo: pixels extent: extent
    "Sets the receiver's bits and extent."

    self bitmapInfo
        width: extent x;
        height: extent y;
        sizeImage: pixels size;
        yourself.
    self bits: pixels.! 
bits: pixels
    "Sets the receiver's bits."

    processor := nil.
    ^super bits: pixels!  
min: min max: max
    "Map the pixels in this image from min-max to 0-255."

	self processor min: min max: max!
or: value
    "Or process on the receiver's color table."

    self processor or: value!   
sizeImage
    " Returns the receiver's size in bytes for dib byteArray."

    ^self processor sizeImage!   
colorTable
        "Private - Answer the receiver color table"

    self mustBeLoaded.
    ^super colorTable! 
flipVertically
    "Flip the receiver vertically."

    self processor flipVertically! 
histogram
    " Returns the receiver's histogram."

    ^self processor histogram! 
resizeTo: extent
    " Resize the receiver to extent."

    self processor resizeTo: extent!   
outputToFile
	"Output the receiver in DIB format to current fileName."

	self outputToFile: self fileName!